# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: voice.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bvoice.proto\x12\x05voice\x1a\x1fgoogle/protobuf/timestamp.proto\"r\n\x13\x45levenLabsTtsConfig\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x18\n\x10similarity_boost\x18\x02 \x01(\x01\x12\x11\n\tstability\x18\x03 \x01(\x01\x12\x12\n\x05speed\x18\x04 \x01(\x01H\x00\x88\x01\x01\x42\x08\n\x06_speed\"\x11\n\x0fPlayHTTtsConfig\"\x85\x01\n\x11\x43\x61rtesiaTtsConfig\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x12\n\x05speed\x18\x02 \x01(\x01H\x00\x88\x01\x01\x12\x14\n\x07\x65motion\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06volume\x18\x04 \x01(\x01H\x02\x88\x01\x01\x42\x08\n\x06_speedB\n\n\x08_emotionB\t\n\x07_volume\"\xa7\x01\n\rHumeTtsConfig\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x1e\n\x11voice_description\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0cinstant_mode\x18\x03 \x01(\x08H\x01\x88\x01\x01\x12\x15\n\x08provider\x18\x04 \x01(\tH\x02\x88\x01\x01\x42\x14\n\x12_voice_descriptionB\x0f\n\r_instant_modeB\x0b\n\t_provider\"\xdf\x02\n\x05Voice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x12\n\ncreated_by\x18\x02 \x01(\t\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nupdated_by\x18\x04 \x01(\t\x12.\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x31\n\x0b\x65leven_labs\x18\x06 \x01(\x0b\x32\x1a.voice.ElevenLabsTtsConfigH\x00\x12)\n\x07play_ht\x18\x07 \x01(\x0b\x32\x16.voice.PlayHTTtsConfigH\x00\x12,\n\x08\x63\x61rtesia\x18\x08 \x01(\x0b\x32\x18.voice.CartesiaTtsConfigH\x00\x12$\n\x04hume\x18\t \x01(\x0b\x32\x14.voice.HumeTtsConfigH\x00\x42\n\n\x08settings\"\xa5\x01\n\nAgentVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x13\n\x0bprobability\x18\x02 \x01(\x01\x12\x12\n\nupdated_by\x18\x03 \x01(\t\x12.\n\nupdated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1a\n\rlanguage_code\x18\x05 \x01(\tH\x00\x88\x01\x01\x42\x10\n\x0e_language_code\"\x95\x01\n\x0f\x44isclaimerVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x12\n\nupdated_by\x18\x02 \x01(\t\x12.\n\nupdated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1a\n\rlanguage_code\x18\x04 \x01(\tH\x00\x88\x01\x01\x42\x10\n\x0e_language_code\"\xbd\x01\n\rFavoriteVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12\x12\n\ncreated_by\x18\x03 \x01(\t\x12.\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nupdated_by\x18\x05 \x01(\t\x12.\n\nupdated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xf0\x01\n\x06Voices\x12\'\n\x0c\x61gent_voices\x18\x01 \x03(\x0b\x32\x11.voice.AgentVoice\x12\x31\n\x11\x64isclaimer_voices\x18\x02 \x03(\x0b\x32\x16.voice.DisclaimerVoice\x12*\n\x14\x61gent_voice_settings\x18\x03 \x03(\x0b\x32\x0c.voice.Voice\x12/\n\x19\x64isclaimer_voice_settings\x18\x04 \x03(\x0b\x32\x0c.voice.Voice\x12-\n\x0f\x66\x61vorite_voices\x18\x05 \x03(\x0b\x32\x14.voice.FavoriteVoice\"j\n\x13Voice_AddAgentVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x13\n\x0bprobability\x18\x02 \x01(\x01\x12\x1a\n\rlanguage_code\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x10\n\x0e_language_code\"*\n\x16Voice_DeleteAgentVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\"k\n\x16Voice_UpdateAgentVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x13\n\x0bprobability\x18\x02 \x01(\x01\x12\x19\n\x0cnew_voice_id\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_new_voice_id\"\xf0\x01\n\x1eVoice_CreateAgentVoiceSettings\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x31\n\x0b\x65leven_labs\x18\x02 \x01(\x0b\x32\x1a.voice.ElevenLabsTtsConfigH\x00\x12)\n\x07play_ht\x18\x07 \x01(\x0b\x32\x16.voice.PlayHTTtsConfigH\x00\x12,\n\x08\x63\x61rtesia\x18\x08 \x01(\x0b\x32\x18.voice.CartesiaTtsConfigH\x00\x12$\n\x04hume\x18\t \x01(\x0b\x32\x14.voice.HumeTtsConfigH\x00\x42\n\n\x08settings\"\xf0\x01\n\x1eVoice_UpdateAgentVoiceSettings\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x31\n\x0b\x65leven_labs\x18\x02 \x01(\x0b\x32\x1a.voice.ElevenLabsTtsConfigH\x00\x12)\n\x07play_ht\x18\x07 \x01(\x0b\x32\x16.voice.PlayHTTtsConfigH\x00\x12,\n\x08\x63\x61rtesia\x18\x08 \x01(\x0b\x32\x18.voice.CartesiaTtsConfigH\x00\x12$\n\x04hume\x18\t \x01(\x0b\x32\x14.voice.HumeTtsConfigH\x00\x42\n\n\x08settings\"]\n\x1bVoice_UpdateDisclaimerVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x1a\n\rlanguage_code\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x10\n\x0e_language_code\"\xf5\x01\n#Voice_CreateDisclaimerVoiceSettings\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x31\n\x0b\x65leven_labs\x18\x02 \x01(\x0b\x32\x1a.voice.ElevenLabsTtsConfigH\x00\x12)\n\x07play_ht\x18\x07 \x01(\x0b\x32\x16.voice.PlayHTTtsConfigH\x00\x12,\n\x08\x63\x61rtesia\x18\x08 \x01(\x0b\x32\x18.voice.CartesiaTtsConfigH\x00\x12$\n\x04hume\x18\t \x01(\x0b\x32\x14.voice.HumeTtsConfigH\x00\x42\n\n\x08settings\"\xf5\x01\n#Voice_UpdateDisclaimerVoiceSettings\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x31\n\x0b\x65leven_labs\x18\x02 \x01(\x0b\x32\x1a.voice.ElevenLabsTtsConfigH\x00\x12)\n\x07play_ht\x18\x07 \x01(\x0b\x32\x16.voice.PlayHTTtsConfigH\x00\x12,\n\x08\x63\x61rtesia\x18\x08 \x01(\x0b\x32\x18.voice.CartesiaTtsConfigH\x00\x12$\n\x04hume\x18\t \x01(\x0b\x32\x14.voice.HumeTtsConfigH\x00\x42\n\n\x08settings\">\n\x16Voice_AddFavoriteVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\"A\n\x19Voice_RemoveFavoriteVoice\x12\x10\n\x08voice_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'voice_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ELEVENLABSTTSCONFIG']._serialized_start=55
  _globals['_ELEVENLABSTTSCONFIG']._serialized_end=169
  _globals['_PLAYHTTTSCONFIG']._serialized_start=171
  _globals['_PLAYHTTTSCONFIG']._serialized_end=188
  _globals['_CARTESIATTSCONFIG']._serialized_start=191
  _globals['_CARTESIATTSCONFIG']._serialized_end=324
  _globals['_HUMETTSCONFIG']._serialized_start=327
  _globals['_HUMETTSCONFIG']._serialized_end=494
  _globals['_VOICE']._serialized_start=497
  _globals['_VOICE']._serialized_end=848
  _globals['_AGENTVOICE']._serialized_start=851
  _globals['_AGENTVOICE']._serialized_end=1016
  _globals['_DISCLAIMERVOICE']._serialized_start=1019
  _globals['_DISCLAIMERVOICE']._serialized_end=1168
  _globals['_FAVORITEVOICE']._serialized_start=1171
  _globals['_FAVORITEVOICE']._serialized_end=1360
  _globals['_VOICES']._serialized_start=1363
  _globals['_VOICES']._serialized_end=1603
  _globals['_VOICE_ADDAGENTVOICE']._serialized_start=1605
  _globals['_VOICE_ADDAGENTVOICE']._serialized_end=1711
  _globals['_VOICE_DELETEAGENTVOICE']._serialized_start=1713
  _globals['_VOICE_DELETEAGENTVOICE']._serialized_end=1755
  _globals['_VOICE_UPDATEAGENTVOICE']._serialized_start=1757
  _globals['_VOICE_UPDATEAGENTVOICE']._serialized_end=1864
  _globals['_VOICE_CREATEAGENTVOICESETTINGS']._serialized_start=1867
  _globals['_VOICE_CREATEAGENTVOICESETTINGS']._serialized_end=2107
  _globals['_VOICE_UPDATEAGENTVOICESETTINGS']._serialized_start=2110
  _globals['_VOICE_UPDATEAGENTVOICESETTINGS']._serialized_end=2350
  _globals['_VOICE_UPDATEDISCLAIMERVOICE']._serialized_start=2352
  _globals['_VOICE_UPDATEDISCLAIMERVOICE']._serialized_end=2445
  _globals['_VOICE_CREATEDISCLAIMERVOICESETTINGS']._serialized_start=2448
  _globals['_VOICE_CREATEDISCLAIMERVOICESETTINGS']._serialized_end=2693
  _globals['_VOICE_UPDATEDISCLAIMERVOICESETTINGS']._serialized_start=2696
  _globals['_VOICE_UPDATEDISCLAIMERVOICESETTINGS']._serialized_end=2941
  _globals['_VOICE_ADDFAVORITEVOICE']._serialized_start=2943
  _globals['_VOICE_ADDFAVORITEVOICE']._serialized_end=3005
  _globals['_VOICE_REMOVEFAVORITEVOICE']._serialized_start=3007
  _globals['_VOICE_REMOVEFAVORITEVOICE']._serialized_end=3072
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: variables.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fvariables.proto\x12\tvariables\x1a\x1fgoogle/protobuf/timestamp.proto\"{\n\x0cVariablesMap\x12\x39\n\tvariables\x18\x01 \x03(\x0b\x32&.variables.VariablesMap.VariablesEntry\x1a\x30\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\xa0\x0c\n\x12VariableReferences\x12?\n\tfunctions\x18\x01 \x03(\x0b\x32,.variables.VariableReferences.FunctionsEntry\x12J\n\x0f\x64\x65lay_responses\x18\x02 \x03(\x0b\x32\x31.variables.VariableReferences.DelayResponsesEntry\x12@\n\nflow_steps\x18\x03 \x03(\x0b\x32,.variables.VariableReferences.FlowStepsEntry\x12N\n\x12\x66low_no_code_steps\x18\x04 \x03(\x0b\x32\x32.variables.VariableReferences.FlowNoCodeStepsEntry\x12H\n\x0e\x66low_functions\x18\x05 \x03(\x0b\x32\x30.variables.VariableReferences.FlowFunctionsEntry\x12\x39\n\x06topics\x18\x06 \x03(\x0b\x32).variables.VariableReferences.TopicsEntry\x12\x41\n\nbehaviours\x18\x07 \x03(\x0b\x32-.variables.VariableReferences.BehavioursEntry\x12?\n\tgreetings\x18\x08 \x03(\x0b\x32,.variables.VariableReferences.GreetingsEntry\x12\x37\n\x05roles\x18\t \x03(\x0b\x32(.variables.VariableReferences.RolesEntry\x12G\n\rpersonalities\x18\n \x03(\x0b\x32\x30.variables.VariableReferences.PersonalitiesEntry\x12\x33\n\x03sms\x18\x0b \x03(\x0b\x32&.variables.VariableReferences.SmsEntry\x12J\n\x0fstart_functions\x18\x0c \x03(\x0b\x32\x31.variables.VariableReferences.StartFunctionsEntry\x12\x46\n\rend_functions\x18\r \x03(\x0b\x32/.variables.VariableReferences.EndFunctionsEntry\x1a\x30\n\x0e\x46unctionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x35\n\x13\x44\x65layResponsesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x30\n\x0e\x46lowStepsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x36\n\x14\x46lowNoCodeStepsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x34\n\x12\x46lowFunctionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a-\n\x0bTopicsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x31\n\x0f\x42\x65havioursEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x30\n\x0eGreetingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a,\n\nRolesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x34\n\x12PersonalitiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a*\n\x08SmsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x35\n\x13StartFunctionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x33\n\x11\x45ndFunctionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\xdf\x01\n\x08Variable\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ncreated_by\x18\x03 \x01(\t\x12.\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nupdated_by\x18\x05 \x01(\t\x12.\n\nupdated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x31\n\nreferences\x18\x07 \x01(\x0b\x32\x1d.variables.VariableReferences\"2\n\tVariables\x12%\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x13.variables.Variable\"^\n\x0fVariable_Create\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x31\n\nreferences\x18\x03 \x01(\x0b\x32\x1d.variables.VariableReferences\"^\n\x0fVariable_Update\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x31\n\nreferences\x18\x03 \x01(\x0b\x32\x1d.variables.VariableReferences\"\x1d\n\x0fVariable_Delete\x12\n\n\x02id\x18\x01 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'variables_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_VARIABLESMAP_VARIABLESENTRY']._options = None
  _globals['_VARIABLESMAP_VARIABLESENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_FUNCTIONSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_FUNCTIONSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_DELAYRESPONSESENTRY']._options = None
  _globals['_VARIABLEREFERENCES_DELAYRESPONSESENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_FLOWSTEPSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_FLOWSTEPSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_FLOWNOCODESTEPSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_FLOWNOCODESTEPSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_FLOWFUNCTIONSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_FLOWFUNCTIONSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_TOPICSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_TOPICSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_BEHAVIOURSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_BEHAVIOURSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_GREETINGSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_GREETINGSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_ROLESENTRY']._options = None
  _globals['_VARIABLEREFERENCES_ROLESENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_PERSONALITIESENTRY']._options = None
  _globals['_VARIABLEREFERENCES_PERSONALITIESENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_SMSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_SMSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_STARTFUNCTIONSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_STARTFUNCTIONSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLEREFERENCES_ENDFUNCTIONSENTRY']._options = None
  _globals['_VARIABLEREFERENCES_ENDFUNCTIONSENTRY']._serialized_options = b'8\001'
  _globals['_VARIABLESMAP']._serialized_start=63
  _globals['_VARIABLESMAP']._serialized_end=186
  _globals['_VARIABLESMAP_VARIABLESENTRY']._serialized_start=138
  _globals['_VARIABLESMAP_VARIABLESENTRY']._serialized_end=186
  _globals['_VARIABLEREFERENCES']._serialized_start=189
  _globals['_VARIABLEREFERENCES']._serialized_end=1757
  _globals['_VARIABLEREFERENCES_FUNCTIONSENTRY']._serialized_start=1094
  _globals['_VARIABLEREFERENCES_FUNCTIONSENTRY']._serialized_end=1142
  _globals['_VARIABLEREFERENCES_DELAYRESPONSESENTRY']._serialized_start=1144
  _globals['_VARIABLEREFERENCES_DELAYRESPONSESENTRY']._serialized_end=1197
  _globals['_VARIABLEREFERENCES_FLOWSTEPSENTRY']._serialized_start=1199
  _globals['_VARIABLEREFERENCES_FLOWSTEPSENTRY']._serialized_end=1247
  _globals['_VARIABLEREFERENCES_FLOWNOCODESTEPSENTRY']._serialized_start=1249
  _globals['_VARIABLEREFERENCES_FLOWNOCODESTEPSENTRY']._serialized_end=1303
  _globals['_VARIABLEREFERENCES_FLOWFUNCTIONSENTRY']._serialized_start=1305
  _globals['_VARIABLEREFERENCES_FLOWFUNCTIONSENTRY']._serialized_end=1357
  _globals['_VARIABLEREFERENCES_TOPICSENTRY']._serialized_start=1359
  _globals['_VARIABLEREFERENCES_TOPICSENTRY']._serialized_end=1404
  _globals['_VARIABLEREFERENCES_BEHAVIOURSENTRY']._serialized_start=1406
  _globals['_VARIABLEREFERENCES_BEHAVIOURSENTRY']._serialized_end=1455
  _globals['_VARIABLEREFERENCES_GREETINGSENTRY']._serialized_start=1457
  _globals['_VARIABLEREFERENCES_GREETINGSENTRY']._serialized_end=1505
  _globals['_VARIABLEREFERENCES_ROLESENTRY']._serialized_start=1507
  _globals['_VARIABLEREFERENCES_ROLESENTRY']._serialized_end=1551
  _globals['_VARIABLEREFERENCES_PERSONALITIESENTRY']._serialized_start=1553
  _globals['_VARIABLEREFERENCES_PERSONALITIESENTRY']._serialized_end=1605
  _globals['_VARIABLEREFERENCES_SMSENTRY']._serialized_start=1607
  _globals['_VARIABLEREFERENCES_SMSENTRY']._serialized_end=1649
  _globals['_VARIABLEREFERENCES_STARTFUNCTIONSENTRY']._serialized_start=1651
  _globals['_VARIABLEREFERENCES_STARTFUNCTIONSENTRY']._serialized_end=1704
  _globals['_VARIABLEREFERENCES_ENDFUNCTIONSENTRY']._serialized_start=1706
  _globals['_VARIABLEREFERENCES_ENDFUNCTIONSENTRY']._serialized_end=1757
  _globals['_VARIABLE']._serialized_start=1760
  _globals['_VARIABLE']._serialized_end=1983
  _globals['_VARIABLES']._serialized_start=1985
  _globals['_VARIABLES']._serialized_end=2035
  _globals['_VARIABLE_CREATE']._serialized_start=2037
  _globals['_VARIABLE_CREATE']._serialized_end=2131
  _globals['_VARIABLE_UPDATE']._serialized_start=2133
  _globals['_VARIABLE_UPDATE']._serialized_end=2227
  _globals['_VARIABLE_DELETE']._serialized_start=2229
  _globals['_VARIABLE_DELETE']._serialized_end=2258
# @@protoc_insertion_point(module_scope)



# Copyright PolyAI Limited
# flake8: noqa
# ruff: noqa
# type: ignore
from typing import Any, Literal, NewType

from runtime.attachment import Attachment
from runtime.conv_utils import Utils
from runtime.external_events import ExternalEvents, SMSReceived
from runtime.history import AgentResponse, UserInput
from runtime.log_utils import ConversationLogger
from runtime.memory import Memory
from runtime.sms import OutgoingSMS, OutgoingSMSTemplate, SMSTemplate

__all__ = [
    "SMSIntegrationNotFound",
    "SMSMissingAssistantAccess",
    "MissingTemplate",
    "TTSVoice",
    "CustomVoice",
    "ElevenLabsVoice",
    "RimeVoice",
    "GoogleVoice",
    "EmotionKindValue",
    "EmotionIntensityValue",
    "Emotion",
    "EmotionIntensity",
    "EmotionKind",
    "CartesiaVoice",
    "PlayHTVoice",
    "MinimaxVoice",
    "HumeVoice",
    "VoiceType",
    "VoiceWeighting",
    "Variant",
    "State",
    "Conversation",
    "MetricEvent",
    "FunctionExecutor",
    "ApiIntegrations",
    "ApiExecutor",
]

# --- Exceptions ---


class SMSIntegrationNotFound(Exception):
    """No integration with given provider found in secret"""

    def __init__(self, secret_name: str, integration: str) -> None: ...


class SMSMissingAssistantAccess(Exception):
    """No access for assistant on SMS secret"""

    def __init__(self, secret_name: str, assistant_id: str, integration: str) -> None: ...


class MissingTemplate(Exception):
    """Template reference doesn't exist"""


# --- Emotion types ---

EmotionKindValue = NewType("EmotionKindValue", int)
EmotionIntensityValue = NewType("EmotionIntensityValue", int)


class EmotionKind:
    """Enum for emotion kind"""

    ANGER: EmotionKindValue
    POSITIVITY: EmotionKindValue
    SURPRISE: EmotionKindValue


class EmotionIntensity:
    """Enum for emotion intensity"""

    LOWEST: EmotionIntensityValue
    LOW: EmotionIntensityValue
    HIGH: EmotionIntensityValue
    HIGHEST: EmotionIntensityValue


class Emotion:
    """Emotion for Cartesia voice"""

    kind: EmotionKindValue | None
    intensity: EmotionIntensityValue | None

    def __init__(
        self,
        kind: EmotionKindValue | None = ...,
        intensity: EmotionIntensityValue | None = ...,
    ) -> None: ...
    def to_dict(self) -> dict: ...


# --- Voice types ---


class TTSVoice:
    """Base class for TTS voice configurations"""

    def __init__(self, provider: str, provider_voice_id: str, config: dict = ...) -> None: ...
    @property
    def provider(self) -> str: ...
    @property
    def provider_voice_id(self) -> str: ...
    def to_dict(self) -> dict: ...


class CustomVoice(TTSVoice):
    """Voice configuration for a custom TTS provider."""

    def __init__(self, provider: str, provider_voice_id: str, **kwargs: Any) -> None: ...


class ElevenLabsVoice(TTSVoice):
    """Voice configuration for ElevenLabs."""

    def __init__(
        self,
        provider_voice_id: str,
        similarity_boost: float | None = ...,
        stability: float | None = ...,
        model_id: Literal[
            "eleven_monolingual_v1",
            "eleven_multilingual_v1",
            "eleven_turbo_v2",
            "eleven_turbo_v2_5",
            "eleven_flash_v2_5",
        ]
        | None = ...,
        speed: float | None = ...,
    ) -> None: ...


class RimeVoice(TTSVoice):
    """Rime voice config"""

    def __init__(
        self,
        provider_voice_id: str,
        speech_alpha: float | None = ...,
        model_id: Literal["mist", "mistv2"] | None = ...,
    ) -> None: ...


class CartesiaVoice(TTSVoice):
    """Cartesia voice config"""

    def __init__(
        self,
        provider_voice_id: str,
        speed: float | None = ...,
        emotions: list[Emotion] | None = ...,
        model_id: str | None = ...,
        volume: float | None = ...,
        emotion: str | None = ...,
        language: str | None = ...,
    ) -> None: ...
    @property
    def emotions(self) -> list[Emotion] | None: ...
    @property
    def speed(self) -> float | None: ...
    @property
    def volume(self) -> float | None: ...
    @property
    def emotion(self) -> str | None: ...
    @property
    def language(self) -> str | None: ...


class PlayHTVoice(TTSVoice):
    """Voice config for PlayHT"""

    def __init__(
        self,
        provider_voice_id: str,
        speed: float | None = ...,
        temperature: float | None = ...,
        emotion: Literal[
            "female_happy",
            "female_sad",
            "female_angry",
            "female_fearful",
            "female_disgust",
            "female_surprised",
            "male_happy",
            "male_sad",
            "male_angry",
            "male_fearful",
            "male_disgust",
            "male_surprised",
        ]
        | None = ...,
        voice_guidance: int | None = ...,
        style_guidance: int | None = ...,
        voice_engine: Literal[
            "Play3.0-mini", "PlayDialog", "PlayHT2.0-turbo", "PlayHT2.0", "PlayHT1.0"
        ]
        | None = ...,
    ) -> None: ...


class MinimaxVoice(TTSVoice):
    """Voice config for Minimax"""

    def __init__(
        self,
        model_id: Literal[
            "speech-02-hd",
            "speech-02-turbo",
            "speech-01-hd",
            "speech-01-turbo",
        ],
        voice_id: str,
        speed: float | None = ...,
        vol: float | None = ...,
        pitch: float | None = ...,
        emotion: Literal["happy", "sad", "angry", "fearful", "disgusted", "surprised", "neutral"]
        | None = ...,
    ) -> None: ...


class HumeVoice(TTSVoice):
    """Voice config for Hume"""

    def __init__(
        self,
        provider_voice_id: str,
        voice_description: str | None = ...,
        version: str | None = ...,
        instant_mode: bool | None = ...,
        provider: Literal["CUSTOM_VOICE", "HUME_AI"] | None = ...,
    ) -> None: ...


class GoogleVoice(TTSVoice):
    """Voice configuration for Google TTS."""

    def __init__(
        self,
        provider_voice_id: str,
        gender: Literal["male", "female", "neutral"] | None = ...,
    ) -> None: ...


VoiceType = (
    CustomVoice
    | ElevenLabsVoice
    | PlayHTVoice
    | CartesiaVoice
    | RimeVoice
    | MinimaxVoice
    | HumeVoice
    | GoogleVoice
)


class VoiceWeighting:
    """Weighting for a voice"""

    def __init__(self, voice: VoiceType, weight: float | None = ...) -> None: ...
    @property
    def voice(self) -> VoiceType: ...
    @property
    def weight(self) -> float | None: ...


# --- State and dict-like types ---


class Variant(dict):
    """Variant object exposing variant attributes"""

    def __getattr__(self, key: str) -> Any: ...


class Entities(dict):
    """Entities object exposing entities attributes"""

    def __getattr__(self, key: str) -> Any: ...


class ReadOnlyDict(dict):
    """Read-only dictionary"""

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...


class RealtimeConfig(ReadOnlyDict):
    """Realtime config"""

    def __init__(self, **kwargs: Any) -> None: ...


class State(dict):
    """`dict` subclass with ergonomic attribute-style access"""

    def __getattr__(self, key: str) -> Any: ...
    def __setattr__(self, key: str, value: Any) -> None: ...


# --- Metric ---


class MetricEvent:
    """Representation of a metric that has already been written to history."""

    name: str
    value: float | str | int | None

    def __init__(self, name: str, value: float | str | int | None) -> None: ...


# --- Executors and integrations ---


class ApiIntegrations:
    """Access to configured API integrations"""

    def __getattr__(self, name: str) -> Any: ...


class FunctionExecutor(dict):
    """Function executor for importing functions from the functions directory."""

    def __init__(self, conv: Conversation) -> None: ...
    def __getattr__(self, name: str) -> Any: ...


class ApiExecutor:
    """API executor for calling configured API integrations."""

    def __init__(
        self,
        conv: Conversation,
        api_integrations: ApiIntegrations | None = ...,
    ) -> None: ...
    def __getattr__(self, name: str) -> Any: ...


# --- Conversation ---


class Conversation:
    """Object exposing useful information from the conversation runtime"""

    memory: Memory | None
    utils: Utils
    log: ConversationLogger

    @property
    def id(self) -> str: ...
    @property
    def account_id(self) -> str: ...
    @property
    def project_id(self) -> str: ...
    @property
    def env(self) -> str: ...
    @property
    def sip_headers(self) -> dict[str, str]: ...
    @property
    def integration_attributes(self) -> dict[str, Any] | None: ...
    @property
    def caller_number(self) -> str | None: ...
    @property
    def callee_number(self) -> str | None: ...
    @property
    def state(self) -> State: ...
    @property
    def entities(self) -> Entities: ...
    @property
    def current_flow(self) -> str | None: ...
    @property
    def current_step(self) -> str | None: ...
    @property
    def sms_queue(self) -> list[OutgoingSMS | OutgoingSMSTemplate]: ...
    @property
    def metrics_queue(self) -> list[dict]: ...
    @property
    def variant_name(self) -> str | None: ...
    @property
    def variants(self) -> dict[str, Variant]: ...
    @property
    def variant(self) -> Variant | None: ...
    @property
    def sms_templates(self) -> dict[str, SMSTemplate]: ...
    @property
    def voice_change(self) -> TTSVoice | None: ...
    @property
    def language(self) -> str | None: ...
    @property
    def history(self) -> list[UserInput | AgentResponse]: ...
    @property
    def handoffs(self) -> dict[str, Any]: ...
    @property
    def transcript_alternatives(self) -> list[str]: ...
    @property
    def real_time_config(self) -> dict[str, Any]: ...
    @property
    def functions(self) -> FunctionExecutor: ...
    @property
    def api(self) -> ApiExecutor: ...
    @property
    def generic_external_events(self) -> list[dict]: ...
    @property
    def channel_type(self) -> str: ...
    @property
    def attachments(self) -> list[Attachment]: ...
    @property
    def response_suggestions(self) -> list[str]: ...
    @property
    def metric_events(self) -> list[MetricEvent]: ...
    def set_voice(self, voice: VoiceType) -> None: ...
    def set_language(self, language: str) -> None: ...
    def set_asr_biasing(
        self,
        keywords: list[str] | None = ...,
        custom_biases: dict[str, float] | None = ...,
    ) -> None: ...
    def clear_asr_biasing(self) -> None: ...
    def say(self, utterance: str) -> None: ...
    def randomize_voice(self, voice_weights: list[VoiceWeighting]) -> None: ...
    def goto_flow(self, flow_name: str) -> None: ...
    def exit_flow(self) -> None: ...
    def goto_csat_flow(self) -> None: ...
    def set_variant(self, variant: str) -> None: ...
    def send_sms(
        self,
        to_number: str,
        from_number: str,
        content: str,
        retry_count: int | None = ...,
    ) -> dict | None: ...
    def send_whatsapp(
        self,
        to_number: str,
        from_number: str,
        content_id: str,
        content: str | None = ...,
        retry_count: int | None = ...,
    ) -> dict | None: ...
    def send_content_template(
        self,
        messaging_service_id: str,
        to_number: str,
        content_id: str,
        content: str | None = ...,
        whatsapp: bool | None = ...,
        content_variables: dict | None = ...,
        retry_count: int | None = ...,
    ) -> dict | None: ...
    def send_sms_template(
        self,
        to_number: str,
        template: str,
        retry_count: int | None = ...,
    ) -> dict | None: ...
    def send_email(self, to: str, body: str, subject: str = ...) -> None: ...
    def call_handoff(
        self,
        destination: str,
        reason: str = ...,
        utterance: str = ...,
        sip_headers: dict[str, str] | None = ...,
        route: str | None = ...,
    ) -> None: ...
    def generate_external_event(self, *, send_to_llm: bool = ...) -> str: ...
    def write_metric(
        self,
        name: str,
        value: float | int | str | None = ...,
        *,
        write_once: bool = ...,
    ) -> None: ...
    def add_attachments(self, attachments: list[Attachment]) -> None: ...
    def set_response_suggestions(self, suggestions: list[str]) -> None: ...
    def set_csat_eligibility(self, eligible: bool, reason: str | None = ...) -> None: ...
    def set_csat_phone_number(self, phone_number: str) -> None: ...
    def set_csat_score(self, score: int) -> None: ...
    def set_csat_survey_entered(self) -> None: ...
    def discard_recording(self) -> None: ...
    def log_api_response(self, response: Any, override_url: str = ...) -> None: ...



"""Sync client for Agent Studio content management

Copyright PolyAI Limited
"""

import logging
import uuid
from copy import deepcopy
from typing import Any, Optional

from poly.handlers.protobuf.commands_pb2 import Command
from poly.handlers.protobuf.handoff_pb2 import Handoff_SetDefault
from poly.handlers.sdk import SourcererAPIError, SourcererSDK
from poly.resources import (
    ApiIntegration,
    ApiIntegrationEnvironments,
    ApiIntegrationOperation,
    ASRBiasing,
    AsrSettings,
    BaseResource,
    ChatGreeting,
    ChatStylePrompt,
    Condition,
    DTMFConfig,
    Entity,
    ExperimentalConfig,
    FlowConfig,
    FlowStep,
    Function,
    FunctionDelayResponse,
    FunctionLatencyControl,
    FunctionParameters,
    FunctionStep,
    FunctionType,
    Handoff,
    KeyphraseBoosting,
    PhraseFilter,
    Pronunciation,
    RegularExpressionRule,
    Resource,
    SettingsPersonality,
    SettingsRole,
    SettingsRules,
    SMSTemplate,
    Topic,
    TranscriptCorrection,
    Variable,
    Variant,
    VariantAttribute,
    VoiceDisclaimerMessage,
    VoiceGreeting,
    VoiceStylePrompt,
)

logger = logging.getLogger(__name__)

ROLE = "role"
PERSONALITY = "personality"
RULES = "rules"


class SyncClientHandler:
    """Sync client for Agent Studio content management"""

    region_to_region_env = {
        "euw-1": ("euw", "prod"),
        "uk-1": ("uk", "prod"),
        "us-1": ("us", "prod"),
        "staging": ("us", "staging"),
        "dev": ("us", "dev"),
    }

    _sdk: Optional[SourcererSDK] = None
    region: str
    account_id: str
    project_id: str

    @property
    def branch_id(self) -> str:
        """Get the current branch ID."""
        return self._sdk.branch_id

    def __init__(
        self,
        region: str,
        account_id: str,
        project_id: str,
        branch_id: Optional[str] = None,
    ):
        region_env = SyncClientHandler.region_to_region_env.get(region)
        if not region_env:
            raise ValueError(f"Unsupported region: {region}")

        self.region = region
        self.account_id = account_id
        self.project_id = project_id

        self._sdk = SourcererSDK(
            region=region_env[0],
            environment=region_env[1],
            account_id=account_id,
            project_id=project_id,
            branch_id=branch_id,
        )

    @property
    def sdk(self) -> SourcererSDK:
        """Get the Sourcerer SDK instance."""
        return self._sdk

    def assert_branch_exists(self) -> str:
        """Assert that the branch exists and switch to 'main' if it doesn't."""
        if self.branch_id != "main":
            found_branches = self._sdk.fetch_branches().get("branches", [])
            branch = next((b for b in found_branches if b.get("branchId") == self.branch_id), None)
            if not branch:
                logger.info(
                    f"Branch ID:'{self.branch_id}' does not exist. Switching to 'main' branch."
                )
                self._sdk.branch_id = "main"
        return self.branch_id

    @classmethod
    def load_resources_from_projection(
        cls, projection: dict
    ) -> dict[type[Resource], dict[str, Resource]]:
        return {
            Topic: cls._read_topics_from_projection(projection),
            Function: cls._read_functions_from_projection(projection),
            Entity: cls._read_entities_from_projection(projection),
            Variable: cls._read_variables_from_projection(projection),
            **cls._read_agent_settings_from_projection(projection),
            **cls._read_channel_settings_from_projection(projection),
            **cls._read_flows_from_projection(projection),
            ExperimentalConfig: cls._read_experimental_config_from_projection(projection),
            SMSTemplate: cls._read_sms_templates_from_projection(projection),
            Handoff: cls._read_handoffs_from_projection(projection),
            **cls._read_variants_from_projection(projection),
            PhraseFilter: cls._read_phrase_filters_from_projection(projection),
            Pronunciation: cls._read_pronunciations_from_projection(projection),
            KeyphraseBoosting: cls._read_keyphrase_boosting_from_projection(projection),
            TranscriptCorrection: cls._read_transcript_corrections_from_projection(projection),
            **cls._read_asr_settings_from_projection(projection),
            ApiIntegration: cls._read_api_integrations_from_projection(projection),
        }  # ty:ignore[invalid-return-type]

    def pull_deployment_resources(
        self, deployment_id: str
    ) -> dict[type[Resource], dict[str, Resource]]:
        """Fetch all resources for a specific deployment of a project.
        Args:
            deployment_id (str): The deployment ID
        Returns:
            dict[type[Resource], dict[str, Resource]]: A dictionary mapping resource types to
                their resources
        """
        logger.info(
            f"Fetching project data for project {self.project_id} for deployment {deployment_id}"
        )
        self.assert_branch_exists()
        projection = self.sdk.fetch_deployment_projection(deployment_id=deployment_id)
        logger.info(
            f"Successfully fetched project data for project {self.project_id} for deployment {deployment_id}"
        )
        return self.load_resources_from_projection(projection)

    def pull_resources(self) -> tuple[dict[type[Resource], dict[str, Resource]], dict[str, Any]]:
        """Fetch all resources from a specific project.

        Returns:
            dict[type[Resource], dict[str, Resource]]: A dictionary mapping resource types to
                their resources
            dict[str, Any]: The projection data
        """
        logger.info(
            f"Fetching project data for project {self.project_id} on branch {self.sdk.branch_id}"
        )
        self.assert_branch_exists()
        projection = self.sdk.fetch_projection(force_refresh=True)
        logger.debug(f"Projection: {projection}")
        logger.info(
            f"Successfully fetched project data for project {self.project_id} on branch {self.sdk.branch_id}"
        )
        return self.load_resources_from_projection(projection), projection

    @staticmethod
    def _read_topics_from_projection(projection: dict) -> dict[str, Topic]:
        topics = {}
        for topic_id, topic in (
            projection.get("knowledgeBase", {}).get("topics", {}).get("entities", {}).items()
        ):
            example_queries = topic.get("exampleQueries", [])
            queries = [
                example_queries["query"]
                for example_queries in example_queries
                if "query" in example_queries
            ]
            topics[topic_id] = Topic(
                resource_id=topic_id,
                name=topic["name"],
                actions=topic["actions"],
                content=topic["content"],
                example_queries=queries,
                enabled=topic.get("isActive", True),
            )

        return topics

    @staticmethod
    def _parse_latency_control(latency_control_data: dict) -> FunctionLatencyControl:
        """Parse latency control from a projection dictionary."""
        if not latency_control_data:
            return FunctionLatencyControl()

        delay_responses = latency_control_data.get(
            "delayResponses", latency_control_data.get("delay_responses", {})
        )
        if isinstance(delay_responses, dict):
            delay_responses = list(
                delay_responses.get("entities", delay_responses).values()
                if "entities" in delay_responses
                else delay_responses.values()
            )

        delay_responses = [
            FunctionDelayResponse(
                id=delay_response.get("id"),
                message=delay_response.get("message", ""),
                duration=delay_response.get("duration", 0),
            )
            for delay_response in delay_responses
            if isinstance(delay_response, dict)
        ]

        return FunctionLatencyControl(
            enabled=latency_control_data.get("enabled", False),
            initial_delay=latency_control_data.get(
                "initialDelay", latency_control_data.get("initial_delay", 0)
            ),
            interval=latency_control_data.get("interval", 0),
            delay_responses=delay_responses,
        )

    @staticmethod
    def _read_functions_from_projection(projection: dict) -> dict[str, Function]:
        functions = {}
        # Read special functions
        special_functions = projection.get("specialFunctions", {})
        for func_type, func in special_functions.items():
            if func.get("archived", False):
                continue

            if func_type == "startFunction":
                func_type = FunctionType.START

            if func_type == "endFunction":
                func_type = FunctionType.END

            functions[func["id"]] = Function(
                resource_id=func["id"],
                name=func["name"],
                description=func["description"],
                code=func["code"],
                parameters=[
                    FunctionParameters(
                        name=parameter.get("name"),
                        type=parameter.get("type"),
                        id=parameter.get("id"),
                        description=parameter.get("description"),
                    )
                    for parameter in func.get("parameters", {}).get("entities", {}).values()
                ],
                latency_control=SyncClientHandler._parse_latency_control(
                    func.get("latencyControl", func.get("latency_control"))
                ),
                flow_id=None,
                function_type=func_type,
            )

        # Read flow functions
        flows = projection.get("flows", {}).get("flows", {}).get("entities", {})
        for flow_id, flow_data in flows.items():
            for func_id, func in (
                flow_data.get("transitionFunctions", {}).get("entities", {}).items()
            ):
                if func.get("archived", False):
                    continue

                functions[func_id] = Function(
                    resource_id=func_id,
                    name=func["name"],
                    description=func["description"],
                    code=func["code"],
                    parameters=[
                        FunctionParameters(
                            name=parameter.get("name"),
                            type=parameter.get("type"),
                            id=parameter.get("id"),
                            description=parameter.get("description"),
                        )
                        for parameter in func.get("parameters", {}).get("entities", {}).values()
                    ],
                    latency_control=SyncClientHandler._parse_latency_control(
                        func.get("latencyControl", func.get("latency_control"))
                    ),
                    flow_id=flow_id,
                    flow_name=flow_data["name"],
                    function_type=FunctionType.TRANSITION,
                )

        # Read regular functions
        for func_id, func in (
            projection.get("functions", {}).get("functions", {}).get("entities", {}).items()
        ):
            if func.get("archived", False):
                continue

            functions[func_id] = Function(
                resource_id=func_id,
                name=func["name"],
                description=func["description"],
                code=func["code"],
                parameters=[
                    FunctionParameters(
                        name=parameter.get("name"),
                        type=parameter.get("type"),
                        id=parameter.get("id"),
                        description=parameter.get("description"),
                    )
                    for parameter in func.get("parameters", {}).get("entities", {}).values()
                ],
                latency_control=SyncClientHandler._parse_latency_control(
                    func.get("latencyControl", func.get("latency_control"))
                ),
                flow_id=None,
                function_type=FunctionType.GLOBAL,
            )

        return functions

    @staticmethod
    def _read_agent_settings_from_projection(
        projection: dict,
    ) -> dict[type[Resource], dict[str, Resource]]:
        settings = {}
        agent_settings = projection.get("agentSettings", {})
        if personality := agent_settings.get("personality", None):
            settings[SettingsPersonality] = {
                PERSONALITY: SettingsPersonality(
                    resource_id=PERSONALITY,
                    name=PERSONALITY,
                    adjectives=personality.get("adjectives", {}),
                    custom=personality.get("custom", ""),
                )
            }
        if role := agent_settings.get("role", None):
            settings[SettingsRole] = {
                ROLE: SettingsRole(
                    resource_id=ROLE,
                    name=ROLE,
                    value=role.get("value", ""),
                    additional_info=role.get("additionalInfo", ""),
                    custom=role.get("custom", ""),
                )
            }
        if rules := agent_settings.get("rules", None):
            settings[SettingsRules] = {
                RULES: SettingsRules(
                    resource_id=RULES,
                    name=RULES,
                    behaviour=rules.get("behaviour", ""),
                )
            }
        return settings

    @staticmethod
    def _read_channel_settings_from_projection(
        projection: dict,
    ) -> dict[type[Resource], dict[str, Resource]]:
        settings = {}
        channels = projection.get("channels", {})

        voice_settings = channels.get("voice", {})
        voice_config = voice_settings.get("config", {}) or {}
        if voice_greeting := voice_config.get("greeting", None):
            settings[VoiceGreeting] = {
                "voice_greeting": VoiceGreeting(
                    resource_id="voice_greeting",
                    name="voice_greeting",
                    welcome_message=voice_greeting.get("welcomeMessage", ""),
                    language_code=voice_greeting.get("languageCode", "en-GB"),
                )
            }
        if voice_style_prompt := voice_config.get("stylePrompt", None):
            settings[VoiceStylePrompt] = {
                "voice_style_prompt": VoiceStylePrompt(
                    resource_id="voice_style_prompt",
                    name="voice_style_prompt",
                    prompt=voice_style_prompt.get("prompt", ""),
                )
            }
        if voice_disclaimer := voice_settings.get("disclaimer", None):
            settings[VoiceDisclaimerMessage] = {
                "voice_disclaimer": VoiceDisclaimerMessage(
                    resource_id="voice_disclaimer",
                    name="voice_disclaimer",
                    message=voice_disclaimer.get("message", ""),
                    enabled=voice_disclaimer.get("enabled", False),
                    language_code=voice_disclaimer.get("languageCode", "en-GB"),
                )
            }

        web_chat_settings = channels.get("webChat", {})
        if not web_chat_settings.get("status", False):
            return settings

        chat_config = web_chat_settings.get("config", {}) or {}
        if chat_greeting := chat_config.get("greeting", None):
            settings[ChatGreeting] = {
                "chat_greeting": ChatGreeting(
                    resource_id="chat_greeting",
                    name="chat_greeting",
                    welcome_message=chat_greeting.get("welcomeMessage", ""),
                    language_code=chat_greeting.get("languageCode", "en-GB"),
                )
            }
        if chat_style_prompt := chat_config.get("stylePrompt", None):
            settings[ChatStylePrompt] = {
                "chat_style_prompt": ChatStylePrompt(
                    resource_id="chat_style_prompt",
                    name="chat_style_prompt",
                    prompt=chat_style_prompt.get("prompt", ""),
                )
            }

        return settings

    @staticmethod
    def _read_entities_from_projection(
        projection: dict,
    ) -> dict[str, Entity]:
        entities = {}
        for entity_id, entity_data in (
            projection.get("entities", {}).get("entities", {}).get("entities", {}).items()
        ):
            entities[entity_id] = Entity(
                resource_id=entity_id,
                name=entity_data["name"],
                description=entity_data.get("description", ""),
                entity_type=entity_data.get("type"),
                config=entity_data.get("config", {}).get("value", {}),
            )
        return entities

    @staticmethod
    def _read_variables_from_projection(
        projection: dict,
    ) -> dict[str, Variable]:
        variables = {}
        variables_data = projection.get("variables", {}).get("variables", {}).get("entities", {})
        for var_id, var_data in variables_data.items():
            variables[var_id] = Variable(
                resource_id=var_id,
                name=var_data.get("name", ""),
            )

        return variables

    @staticmethod
    def _read_flows_from_projection(
        projection: dict,
    ) -> dict[type[Resource], dict[str, Resource]]:
        resources: dict[type[Resource], dict[str, Resource]] = {}
        flows = projection.get("flows", {}).get("flows", {}).get("entities", {})

        for flow_id, flow_data in flows.items():
            resources.setdefault(FlowConfig, {})[flow_id] = FlowConfig(
                resource_id=flow_id,
                name=flow_data["name"],
                description=flow_data.get("description", ""),
                start_step=flow_data.get("startStepId", ""),
            )

            for step_id, step in flow_data.get("steps", {}).get("entities", {}).items():
                local_resource_id = f"{flow_data['name']}_{step_id}"

                if step.get("type") == "function_step":
                    function = step.get("function", {})
                    resources.setdefault(FunctionStep, {})[local_resource_id] = FunctionStep(
                        resource_id=local_resource_id,
                        step_id=step_id,
                        flow_id=flow_id,
                        flow_name=flow_data["name"],
                        name=step["name"],
                        position=step.get("position"),
                        code=function.get("code", ""),
                        latency_control=SyncClientHandler._parse_latency_control(
                            function.get("latencyControl", function.get("latency_control"))
                        ),
                        parameters=[],
                        function_id=function.get("id", ""),
                    )
                    continue

                asr_biasing_data = step.get("asrBiasing", {})
                dtmf_config_data = step.get("dtmfConfig", {})

                references = step.get("references", {})
                extracted_entities = list(references.get("extractedEntities", {}).keys())

                resources.setdefault(FlowStep, {})[local_resource_id] = FlowStep(
                    resource_id=local_resource_id,
                    step_id=step_id,
                    name=step["name"],
                    flow_id=flow_id,
                    flow_name=flow_data["name"],
                    step_type=step.get("type"),
                    asr_biasing=ASRBiasing(
                        alphanumeric=asr_biasing_data.get("alphanumeric", False),
                        name_spelling=asr_biasing_data.get("nameSpelling", False),
                        numeric=asr_biasing_data.get("numeric", False),
                        party_size=asr_biasing_data.get("partySize", False),
                        precise_date=asr_biasing_data.get("preciseDate", False),
                        relative_date=asr_biasing_data.get("relativeDate", False),
                        single_number=asr_biasing_data.get("singleNumber", False),
                        time=asr_biasing_data.get("time", False),
                        yes_no=asr_biasing_data.get("yesNo", False),
                        address=asr_biasing_data.get("address", False),
                        custom_keywords=asr_biasing_data.get("customKeywords", []),
                        is_enabled=asr_biasing_data.get("isEnabled", False),
                        step_id=step_id,
                        flow_id=flow_id,
                    ),
                    dtmf_config=DTMFConfig(
                        is_enabled=dtmf_config_data.get("isEnabled", False),
                        inter_digit_timeout=dtmf_config_data.get("interDigitTimeout", 0),
                        max_digits=dtmf_config_data.get("maxDigits", 0),
                        end_key=dtmf_config_data.get("endKey", ""),
                        collect_while_agent_speaking=dtmf_config_data.get(
                            "collectWhileAgentSpeaking", False
                        ),
                        is_pii=dtmf_config_data.get("isPii", False),
                        step_id=step_id,
                        flow_id=flow_id,
                    ),
                    prompt=step.get("prompt", ""),
                    conditions=[
                        Condition(
                            resource_id=condition_data["id"],
                            name=condition_data["config"]["value"]["details"]["label"],
                            condition_type=condition_data["config"]["$case"],
                            description=condition_data["config"]["value"]["details"].get(
                                "description", ""
                            ),
                            required_entities=condition_data["config"]["value"]["details"].get(
                                "requiredEntities", []
                            ),
                            child_step=condition_data["config"]["value"].get("childStepId", ""),
                            step_id=step_id,
                            flow_id=flow_id,
                            ingress=condition_data["config"]["value"]["details"].get(
                                "ingressPosition", "top"
                            ),
                            position=condition_data["config"]["value"]["details"].get(
                                "position", {"x": 0.0, "y": 0.0}
                            ),
                            exit_flow_position=condition_data["config"]["value"].get(
                                "exitFlowPosition", None
                            ),
                        )
                        for condition_data in step.get("conditions", [])
                    ],
                    position=step.get("position"),
                    extracted_entities=extracted_entities,
                )
        return resources

    @staticmethod
    def _read_experimental_config_from_projection(
        projection: dict,
    ) -> dict[str, ExperimentalConfig]:
        experimental_configs = (
            projection.get("experimentalConfig", {})
            .get("experimentalConfigs", {})
            .get("entities", {})
        )
        # Only get the default experimental config
        default_experimental_config = experimental_configs.get("default", {})
        return {
            "default": ExperimentalConfig(
                resource_id="default",
                name="experimental_config",
                config=default_experimental_config.get("features", {}),
            )
        }

    @staticmethod
    def _read_sms_templates_from_projection(
        projection: dict,
    ) -> dict[str, SMSTemplate]:
        sms_templates_projection = (
            projection.get("sms", {}).get("templates", {}).get("entities", {})
        )
        sms_templates = {}
        for sms_template_id, sms_template_data in sms_templates_projection.items():
            if not sms_template_data.get("active", False):
                continue
            sms_templates[sms_template_id] = SMSTemplate(
                resource_id=sms_template_id,
                name=sms_template_data["name"],
                text=sms_template_data.get("text", ""),
                env_phone_numbers=sms_template_data.get("envPhoneNumbers", {}),
            )
        return sms_templates

    @staticmethod
    def _read_handoffs_from_projection(
        projection: dict,
    ) -> dict[str, Handoff]:
        handoffs_projection = projection.get("handoff", {}).get("handoffs", {}).get("entities", {})
        handoffs = {}
        for handoff_id, handoff_data in handoffs_projection.items():
            if not handoff_data.get("active", False):
                continue

            config = handoff_data.get("sipConfig", {}).get("config", {})
            method = config.get("$case", "bye")
            value = config.get("value", {})

            sip_config = {"method": method}
            if method == "invite":
                sip_config["phone_number"] = value.get("phoneNumber", "")
                sip_config["outbound_endpoint"] = value.get("outboundEndpoint", "")
                sip_config["outbound_encryption"] = value.get("outboundEncryption", "")
            elif method == "refer":
                sip_config["phone_number"] = value.get("phoneNumber", "")

            sip_headers = handoff_data.get("sipHeaders", {}).get("headers", [])

            handoffs[handoff_id] = Handoff(
                resource_id=handoff_id,
                name=handoff_data.get("name", ""),
                description=handoff_data.get("description", ""),
                is_default=handoff_data.get("isDefault", False),
                sip_config=sip_config,
                sip_headers=sip_headers,
            )
        return handoffs

    @staticmethod
    def _read_variants_from_projection(
        projection: dict,
    ) -> dict[type[Resource], dict[str, Resource]]:
        response = {}
        variants = {}
        for variant_id, variant_data in (
            projection.get("variantManagement", {}).get("variants", {}).get("entities", {}).items()
        ):
            variants[variant_id] = Variant(
                resource_id=variant_id,
                name=variant_data["name"],
                is_default=variant_data.get("isDefault", False),
            )
        if variants:
            response[Variant] = variants

        # variantAttributeValues: entities: dict[variant_id: dict[attribute_id: attribute_value]]
        # attributes: entities: dict[attribute_id: dict["name": name]
        variant_attributes = {}
        for attribute_id, attribute_data in (
            projection.get("variantManagement", {})
            .get("attributes", {})
            .get("entities", {})
            .items()
        ):
            if attribute_data["archived"]:
                continue
            variant_attributes[attribute_id] = VariantAttribute(
                resource_id=attribute_id, name=attribute_data["name"], mappings={}
            )
        if not variant_attributes:
            return response

        for variant_id, variant_attribute_values in (
            projection.get("variantManagement", {})
            .get("variantAttributeValues", {})
            .get("entities", {})
            .items()
        ):
            for attribute_id, attribute_value in variant_attribute_values.get("values", {}).items():
                if attribute_id in variant_attributes:
                    variant_attributes[attribute_id].mappings[variant_id] = attribute_value

        response[VariantAttribute] = variant_attributes

        return response

    @staticmethod
    def _read_phrase_filters_from_projection(
        projection: dict,
    ) -> dict[str, PhraseFilter]:
        phrase_filters = {}
        for filter_id, filter_data in (
            projection.get("stopKeywords", {}).get("filters", {}).get("entities", {}).items()
        ):
            references = filter_data.get("references", {})
            global_functions = references.get("globalFunctions", {})
            function_id = next(iter(global_functions), None) if global_functions else None

            phrase_filters[filter_id] = PhraseFilter(
                resource_id=filter_id,
                name=filter_data.get("title", ""),
                description=filter_data.get("description", ""),
                regular_expressions=filter_data.get("regularExpressions", []),
                say_phrase=filter_data.get("sayPhrase", False),
                language_code=filter_data.get("languageCode", ""),
                function=function_id,
            )
        return phrase_filters

    @staticmethod
    def _read_pronunciations_from_projection(
        projection: dict,
    ) -> dict[str, Pronunciation]:
        pronunciations = {}
        index = 0
        for pronunciation_id, pronunciation_data in (
            projection.get("pronunciations", {})
            .get("pronunciations", {})
            .get("entities", {})
            .items()
        ):
            # FIXME: Sourcerer SDK bug
            # Sometimes, even if the dict has multiple elements
            # They are all at position 0.
            position = pronunciation_data.get("position") or index
            pronunciations[pronunciation_id] = Pronunciation(
                resource_id=pronunciation_id,
                name=pronunciation_data.get("name", ""),
                regex=pronunciation_data.get("regex", ""),
                replacement=pronunciation_data.get("replacement", ""),
                case_sensitive=pronunciation_data.get("caseSensitive", False),
                language_code=pronunciation_data.get("languageCode", ""),
                description=pronunciation_data.get("description", ""),
                position=position if position == index else index + 1,
            )
            index += 1
        return pronunciations

    @staticmethod
    def _read_keyphrase_boosting_from_projection(
        projection: dict,
    ) -> dict[str, KeyphraseBoosting]:
        keyphrases = {}
        for kp_id, kp_data in (
            projection.get("keyphraseBoosting", {})
            .get("keyphraseBoosting", {})
            .get("entities", {})
            .items()
        ):
            # Names aren't a thing for keyphrase boosting
            keyphrases[kp_id] = KeyphraseBoosting(
                resource_id=kp_id,
                name=kp_data.get("keyphrase", ""),
                keyphrase=kp_data.get("keyphrase", ""),
                level=kp_data.get("level", "default"),
            )
        return keyphrases

    @staticmethod
    def _read_transcript_corrections_from_projection(
        projection: dict,
    ) -> dict[str, TranscriptCorrection]:
        corrections = {}
        for correction_id, correction_data in (
            projection.get("transcriptCorrections", {})
            .get("transcriptCorrections", {})
            .get("entities", {})
            .items()
        ):
            regular_expressions = [
                RegularExpressionRule(
                    regular_expression=r.get("regularExpression", ""),
                    replacement=r.get("replacement", ""),
                    replacement_type=r.get("replacementType", "full"),
                )
                for r in correction_data.get("regularExpressions", [])
            ]
            corrections[correction_id] = TranscriptCorrection(
                resource_id=correction_id,
                name=correction_data.get("name", ""),
                description=correction_data.get("description", ""),
                regular_expressions=regular_expressions,
            )
        return corrections

    @staticmethod
    def _read_asr_settings_from_projection(
        projection: dict,
    ) -> dict[type[Resource], dict[str, Resource]]:
        asr_settings_data = projection.get("channels", {}).get("voice", {}).get("asrSettings", {})
        if not asr_settings_data:
            return {}

        barge_in = asr_settings_data.get("bargeIn", False)

        # Latency Config only has interaction style at this time
        interaction_style = asr_settings_data.get("latencyConfig", {}).get(
            "interactionStyle", "balanced"
        )

        return {
            AsrSettings: {
                "asr_settings": AsrSettings(
                    resource_id="asr_settings",
                    name="asr_settings",
                    barge_in=barge_in,
                    interaction_style=interaction_style,
                )
            }
        }

    @staticmethod
    def _read_api_integrations_from_projection(
        projection: dict,
    ) -> dict[type[Resource], dict[str, Resource]]:
        api_integrations = {}
        for integration_id, integration_data in (
            projection.get("apiIntegrations", {})
            .get("apiIntegrations", {})
            .get("entities", {})
            .items()
        ):
            environments = ApiIntegrationEnvironments.from_dict(
                integration_data.get("environments")
            )
            operations_raw = integration_data.get("operations") or {}
            operations = [ApiIntegrationOperation.from_dict(v) for v in operations_raw.values()]

            api_integrations[integration_id] = ApiIntegration(
                resource_id=integration_id,
                name=integration_data.get("name", ""),
                description=integration_data.get("description", ""),
                environments=environments,
                operations=operations,
            )

        return api_integrations

    # Types that should be created first
    # as they are referenced by other resources
    PRIORITY_CREATE_TYPES = [
        # Things that are referenced in prompts
        Variable,
        Entity,
        Variant,
        VariantAttribute,
        SMSTemplate,
        Handoff,
        Function,
        # Steps should be created before conditions
        FlowConfig,  # Flow config contains initial steps and functions
        FunctionStep,
        FlowStep,
        Condition,
        # Integrations should be created before operations and environments
        ApiIntegration,
    ]

    PRIORITY_DELETE_TYPES = [
        # If a function is deleted and it is the last reference to a variable,
        # the variable will be deleted as well. So we need to delete variables
        # first or its delete command will fail.
        Variable,
    ]

    PRIORITY_UPDATE_TYPES = [
        # If variable references will change, we should update the variable first so
        # it isn't pruned by the backend.
        Variable,
    ]

    def queue_resources(
        self,
        deleted_resources: dict[type[BaseResource], dict[str, BaseResource]],
        new_resources: dict[type[BaseResource], dict[str, BaseResource]],
        updated_resources: dict[type[BaseResource], dict[str, BaseResource]],
        email: Optional[str] = None,
    ) -> list[Command]:
        """Queue multiple resources for the specific project.

        Sends in order:
        - delete
        - create
        - update

        Args:
            deleted_resources (dict[type[BaseResource], dict[str, BaseResource]]): Resources to delete
            new_resources (dict[type[BaseResource], dict[str, BaseResource]]): New resources to upload
            updated_resources (dict[type[BaseResource], dict[str, BaseResource]]): Updated resources to upload
            email (str): Email to use for metadata creation.

        Returns:
            list[Command]: A list of queued Command protobuf messages.
        """
        metadata = self.sdk.create_metadata()
        if email:
            metadata.created_by = email

        commands = []

        delete_resources_priority: list[type[BaseResource]] = []
        for resource_type in self.PRIORITY_DELETE_TYPES:
            if resource_type in deleted_resources:
                delete_resources_priority.append(resource_type)
        for resource_type in deleted_resources.keys():
            if resource_type not in self.PRIORITY_DELETE_TYPES:
                delete_resources_priority.append(resource_type)

        for resource_type in delete_resources_priority:
            for resource_id, resource in deleted_resources.get(resource_type, {}).items():
                delete_type = resource.delete_command_type
                commands.append(
                    Command(
                        type=delete_type,
                        command_id=str(uuid.uuid4()),
                        metadata=metadata,
                        **{delete_type: resource.build_delete_proto()},
                    )
                )

        new_resources_priority: list[type[BaseResource]] = []
        for resource_type in self.PRIORITY_CREATE_TYPES:
            if resource_type in new_resources:
                new_resources_priority.append(resource_type)
        for resource_type in new_resources.keys():
            if resource_type not in self.PRIORITY_CREATE_TYPES:
                new_resources_priority.append(resource_type)

        for resource_type in new_resources_priority:
            resources = new_resources.get(resource_type, {})
            for resource_id, resource in resources.items():
                create_type = resource.create_command_type
                commands.append(
                    Command(
                        type=create_type,
                        command_id=str(uuid.uuid4()),
                        metadata=metadata,
                        **{create_type: resource.build_create_proto()},
                    )
                )

        updated_resource_priority: list[type[BaseResource]] = []
        for resource_type in self.PRIORITY_UPDATE_TYPES:
            if resource_type in updated_resources:
                updated_resource_priority.append(resource_type)
        for resource_type in updated_resources.keys():
            if resource_type not in self.PRIORITY_UPDATE_TYPES:
                updated_resource_priority.append(resource_type)

        for resource_type in updated_resource_priority:
            resources = updated_resources.get(resource_type, {})
            for resource_id, resource in resources.items():
                update_type = resource.update_command_type
                commands.append(
                    Command(
                        type=update_type,
                        command_id=str(uuid.uuid4()),
                        metadata=metadata,
                        **{update_type: resource.build_update_proto()},
                    )
                )

        # is_default is not part of create/update protos; it requires a separate command
        for resource_dict in [new_resources, updated_resources]:
            for resource_id, resource in resource_dict.get(Handoff, {}).items():
                if isinstance(resource, Handoff) and resource.is_default:
                    commands.append(
                        Command(
                            type="handoff_set_default",
                            command_id=str(uuid.uuid4()),
                            metadata=metadata,
                            handoff_set_default=Handoff_SetDefault(id=resource.resource_id),
                        )
                    )

        for command in commands:
            self.sdk.add_command_to_queue(command)

        logger.info(f"Queued {len(commands)} commands")
        logger.debug(f"Commands: {commands!r}")
        return commands

    def send_queued_commands(self) -> bool:
        """Send all queued commands as a batch and clear the queue.

        Returns:
            bool: True if the commands were sent successfully, False otherwise
        """
        if self.sdk.get_queue_size() == 0:
            logger.info("No commands to send")
            return True

        self.assert_branch_exists()

        # Creates branch and switches to it
        if self.sdk.branch_id == "main":
            self.create_branch()

        try:
            logger.info(f"Sending {len(self.sdk._command_queue)} commands to {self.sdk.branch_id}")
            self.sdk.send_command_batch()
            return True
        except SourcererAPIError as e:
            logger.error(f"Failed to send commands: {e}")
            return False

    def clear_command_queue(self) -> None:
        """Clear all queued commands without sending."""
        logger.info(f"Clearing {len(self.sdk._command_queue)} commands")
        self.sdk.clear_queue()

    def get_queued_commands(self) -> list[Command]:
        """Get all queued commands.

        Returns:
            list[Command]: A list of queued Command protobuf messages.
        """
        return deepcopy(self.sdk._command_queue)

    def switch_branch(self, branch_id: str) -> bool:
        """Switch to a different branch within the same project.

        Args:
            branch_id (str): The ID of the branch to switch to

        Returns:
            bool: True if the switch was successful, False otherwise
        """
        self.assert_branch_exists()

        if self.sdk.branch_id == branch_id:
            logger.info(f"Already on branch ID:'{branch_id}'")
            return True

        if branch_id == "main":
            self.sdk.branch_id = "main"
            self.sdk.get_project_data()
            logger.info(f"Switched to branch ID:'{branch_id}'")
            return True

        if found_branches := self.sdk.fetch_branches().get("branches"):
            branch = next((b for b in found_branches if b.get("branchId") == branch_id), None)
            if branch:
                self.sdk.branch_id = branch_id
                # Re-fetch project data to ensure the SDK is up-to-date
                self.sdk.clear_cache()
                self.sdk.get_project_data()
                logger.info(f"Switched to branch ID:'{branch_id}'")
                return True
            else:
                logger.error(f"Branch ID:'{branch_id}' does not exist.")
                return False
        return False

    def create_branch(self, branch_name: Optional[str] = None) -> str:
        """Create a new branch for the project

        Args:
            branch_name: Optional name for the new branch. If not provided, a default name will be used.

        Returns:
            The ID of the created branch
        """
        self.sdk.branch_id = "main"
        self.sdk.get_project_data()

        if branch_name is None:
            metadata = self.sdk.create_metadata()
            time_suffix = f"{metadata.created_at.seconds % 100000:05d}"
            random_suffix = uuid.uuid4().hex[:4]
            suffix = f"{time_suffix}-{random_suffix}"  # to avoid duplicate names
            branch_name = f"ADK-{suffix}"

        logger.info(f"Creating new branch '{branch_name}' from 'main' branch")

        self.sdk.branch_id = self.sdk.create_branch(
            expected_main_last_known_sequence=self.sdk._last_known_sequence,
            branch_name=branch_name,
        )
        logger.info(
            f"Created and switched to new branch. Name:'{branch_name}' ID:'{self.sdk.branch_id}'"
        )
        return self.sdk.branch_id

    def get_branches(self) -> dict[str, str]:
        """Get a list of all branches in the project.

        Returns:
            dict[str, str]: A dictionary mapping branch names to branch IDs
        """
        branches = {"main": "main"}
        logger.info(f"Fetching branches for project {self.account_id}/{self.project_id}")
        for branch in self.sdk.fetch_branches().get("branches"):
            branches[branch.get("name")] = branch.get("branchId")
        logger.info(f"Fetched {len(branches)} branches branches={branches!r}")
        return branches

    def delete_branch(self, branch_id):
        """Delete a branch in the project.

        Args:
            branch_id (str): The ID of the branch to delete

        Returns:
            bool: True if the branch was deleted successfully, False otherwise
        """
        if branch_id == "main":
            logger.error("Cannot delete 'main' branch.")
            return False

        logger.info(f"Deleting branch ID:'{branch_id}'")

        try:
            self.sdk.delete_branch(branch_id=branch_id)
        except SourcererAPIError as e:
            logger.error(f"Failed to delete branch ID:'{branch_id}': {e}")
            return False

        logger.info(f"Successfully deleted branch ID:'{branch_id}'")
        return True

    def merge_branch(
        self, message: str, conflict_resolutions: Optional[list[dict[str, Any]]] = None
    ) -> tuple[bool, list[dict[str, str]], list[dict[str, str]]]:
        """Merge the current branch into main.

        Args:
            message (str): The merge commit message
            conflict_resolutions (list[dict[str, Any]]): A list of conflict resolutions. Each resolution should have:
                - path: List of strings representing the path to the conflicted field (e.g., ["users", "1", "name"])
                - strategy: Resolution strategy - "ours", "theirs", or "base"
                - value: Optional custom value (only used with custom strategy)

        Returns:
            success (bool): True if the merge was successful, False otherwise
            list[dict[str, str]]: A list of conflict information if the merge failed, empty list if successful
            list[dict[str, str]]: A list of error information if the merge failed, empty list if successful
        """
        self.assert_branch_exists()

        if self.sdk.branch_id == "main":
            logger.error("Cannot merge 'main' branch into itself.")
            return False, [], []

        logger.info(f"Merging branch '{self.sdk.branch_id}' into 'main'")

        try:
            result = self.sdk.merge_branch(
                expected_branch_last_known_sequence=self.sdk._last_known_sequence,
                deployment_message=message,
                conflict_resolutions=conflict_resolutions,
            )
        except SourcererAPIError as e:
            logger.error(f"Failed to merge branch '{self.sdk.branch_id}' into 'main': {e}")
            return False, [], []

        if result.get("hasConflicts", False) or result.get("errors", []):
            logger.error(
                f"Failed to merge branch '{self.sdk.branch_id}' into 'main' due to {len(result.get('conflicts', []))} conflicts and {len(result.get('errors', []))} errors"
            )
            conflicts = result.get("conflicts", [])
            errors = result.get("errors", [])
            return False, conflicts, errors

        logger.info(f"Successfully merged branch '{self.sdk.branch_id}' into 'main'")
        return True, [], []

    def get_branch_chat_info(self, branch_id: str) -> dict[str, Any]:
        """Get deployment info needed to start a draft chat on a branch."""
        self.assert_branch_exists()
        return self.sdk.get_branch_chat_info(branch_id)



"""Handling and managing an Agent Studio AgentSettings

Copyright PolyAI Limited
"""

import os
from dataclasses import dataclass, field
from functools import cached_property

from google.protobuf.message import Message

import poly.resources.resource_utils as utils
from poly.handlers.protobuf.agent_settings_pb2 import (
    Personality_UpdatePersonality,
    Role_UpdateRole,
    Rules_UpdateRules,
)
from poly.resources.resource import (
    Resource,
    ResourceMapping,
    YamlResource,
)

ALLOWED_BEHAVIOUR_REFERENCES = ["global_functions", "sms", "handoff", "attributes", "variables"]
ALLOWED_ADJECTIVES = {
    "Polite",
    "Calm",
    "Kind",
    "Funny",
    "Other",
    "Energetic",
    "Thoughtful",
}


@dataclass
class SettingsPersonality(YamlResource):
    """Resource class for managing personality settings"""

    custom: str
    adjectives: dict[str, bool] = field(default_factory=dict)

    @cached_property
    def file_path(self) -> str:
        """Get the file path for the Personality resource."""
        return os.path.join("agent_settings", "personality.yaml")

    def to_yaml_dict(self) -> dict:
        """Convert the personality settings to a YAML-serializable dict."""
        return {"adjectives": self.adjectives, "custom": self.custom}

    @classmethod
    def to_pretty_dict(
        cls, d: dict, resource_mappings: list[ResourceMapping] = None, **kwargs
    ) -> dict:
        """Return the pretty dictionary."""
        d["custom"] = utils.replace_resource_ids_with_names(d["custom"], resource_mappings or [])
        return d

    @classmethod
    def from_yaml_dict(
        cls, yaml_dict: dict, resource_id: str, name: str, **kwargs
    ) -> "SettingsPersonality":
        """Construct personality settings from YAML data and identity fields."""
        return SettingsPersonality(
            resource_id=resource_id,
            name=name,
            adjectives=yaml_dict.get("adjectives", {}),
            custom=yaml_dict.get("custom", ""),
        )

    def validate(self, resource_mappings: list[ResourceMapping] = None, **kwargs) -> None:
        """Validate the personality resource."""
        # Other adjective can only be set if no other adjectives are selected
        if self.adjectives.get("Other", False) and any(
            v for k, v in self.adjectives.items() if k.lower() != "other"
        ):
            raise ValueError("Other adjective can only be set if no other adjectives are selected.")

        # Adjectives must be from the allowed set
        if not set(self.adjectives.keys()).issubset(ALLOWED_ADJECTIVES):
            raise ValueError(
                f"Adjectives must be from the allowed set: {', '.join(ALLOWED_ADJECTIVES)}"
            )

        references = utils.get_references_from_prompt(
            self.custom, ["attributes", "variables"], raise_on_invalid=True
        )
        valid, invalid_references = utils.validate_references(references, resource_mappings)
        if not valid:
            raise ValueError(f"Invalid references: {invalid_references}")

    def build_update_proto(self) -> Personality_UpdatePersonality:
        """Create a proto for updating the resource."""

        return Personality_UpdatePersonality(
            adjectives={
                "values": self.adjectives,
            },
            custom=self.custom,
        )

    def build_create_proto(self) -> Message:
        """Create a proto for creating the resource."""
        raise NotImplementedError("Create operation not supported for Personality settings.")

    def build_delete_proto(self) -> Message:
        """Create a proto for deleting the resource."""
        raise NotImplementedError("Delete operation not supported for Personality settings.")

    @property
    def command_type(self) -> str:
        """Get the update type for updating the resource."""
        return "personality"

    @staticmethod
    def discover_resources(base_path: str) -> list[str]:
        """Discover resources of this type in the given base path.

        Args:
            base_path (str): The base path to search for resources.

        Returns:
            list[str]: A list of file paths of discovered resources.
        """
        file_path = os.path.join(base_path, "agent_settings", "personality.yaml")

        if not os.path.exists(file_path):
            return []

        return [file_path]


@dataclass
class SettingsRole(YamlResource):
    """Resource class for managing role settings"""

    value: str
    additional_info: str
    custom: str

    @cached_property
    def file_path(self) -> str:
        """Get the file path for the Role resource."""
        return os.path.join("agent_settings", "role.yaml")

    def to_yaml_dict(self) -> dict:
        """Convert the role settings to a YAML-serializable dict."""
        return {
            "value": self.value,
            "additional_info": self.additional_info,
            "custom": self.custom,
        }

    @classmethod
    def to_pretty_dict(
        cls, d: dict, resource_mappings: list[ResourceMapping] = None, **kwargs
    ) -> dict:
        """Return the pretty dictionary."""
        d["custom"] = utils.replace_resource_ids_with_names(d["custom"], resource_mappings or [])
        return d

    @classmethod
    def from_yaml_dict(
        cls, yaml_dict: dict, resource_id: str, name: str, **kwargs
    ) -> "SettingsRole":
        """Construct role settings from YAML data and identity fields."""
        return SettingsRole(
            resource_id=resource_id,
            name=name,
            value=yaml_dict.get("value", ""),
            additional_info=yaml_dict.get("additional_info", ""),
            custom=yaml_dict.get("custom", ""),
        )

    def validate(self, resource_mappings: list[ResourceMapping] = None, **kwargs) -> None:
        """Validate the role resource."""
        # Custom role can only exist if role is 'other'
        if self.custom and self.value.lower() != "other":
            raise ValueError("Custom role can only be set if role is 'other'.")

        # Validate references
        references = utils.get_references_from_prompt(
            self.custom, ["attributes", "variables"], raise_on_invalid=True
        )
        valid, invalid_references = utils.validate_references(references, resource_mappings)
        if not valid:
            raise ValueError(f"Invalid references: {invalid_references}")

    def build_update_proto(self) -> Role_UpdateRole:
        """Create a proto for updating the resource."""

        return Role_UpdateRole(
            value=self.value,
            additional_info=self.additional_info,
            custom=self.custom,
        )

    def build_create_proto(self) -> Message:
        """Create a proto for creating the resource."""
        raise NotImplementedError("Create operation not supported for Role settings.")

    def build_delete_proto(self) -> Message:
        """Create a proto for deleting the resource."""
        raise NotImplementedError("Delete operation not supported for Role settings.")

    @property
    def command_type(self) -> str:
        """Get the update type for updating the resource."""
        return "role"

    @staticmethod
    def discover_resources(base_path: str) -> list[str]:
        """Discover resources of this type in the given base path.

        Args:
            base_path (str): The base path to search for resources.

        Returns:
            list[str]: A list of file paths of discovered resources.
        """
        file_path = os.path.join(base_path, "agent_settings", "role.yaml")

        if not os.path.exists(file_path):
            return []

        return [file_path]


@dataclass
class SettingsRules(Resource):
    """Resource class for managing rules settings"""

    behaviour: str

    @cached_property
    def file_path(self) -> str:
        """Get the file path for the Rules resource."""
        return os.path.join("agent_settings", "rules.txt")

    @property
    def raw(self) -> str:
        """Convert the resource to a raw format."""
        return self.behaviour

    @staticmethod
    def make_pretty(
        contents: str, resource_mappings: list[ResourceMapping] = None, **kwargs
    ) -> str:
        """Replace resource IDs with resource names in the provided contents."""
        return utils.replace_resource_ids_with_names(contents, resource_mappings or [])

    @staticmethod
    def from_pretty(
        contents: str, resource_mappings: list[ResourceMapping] = None, **kwargs
    ) -> str:
        """Replace resource names with resource IDs in the provided contents."""
        return utils.replace_resource_names_with_ids(contents, resource_mappings or [])

    def validate(self, resource_mappings: list[ResourceMapping] = None, **kwargs) -> None:
        """Validate the rules resource."""
        references = utils.get_references_from_prompt(
            self.behaviour, ALLOWED_BEHAVIOUR_REFERENCES, raise_on_invalid=True
        )
        valid, invalid_references = utils.validate_references(references, resource_mappings)
        if not valid:
            raise ValueError(f"Invalid references: {invalid_references}")

    @classmethod
    def read_local_resource(
        cls, file_path: str, resource_id: str, resource_name: str, **kwargs
    ) -> "SettingsRules":
        """Read a local YAML resource from the given file path."""
        content = cls.read_to_raw(file_path, **kwargs)
        behaviour = content
        return SettingsRules(
            resource_id=resource_id,
            name=resource_name,
            behaviour=behaviour,
        )

    def build_update_proto(self) -> Rules_UpdateRules:
        """Create a proto for updating the resource."""

        references = utils.get_references_from_prompt(self.behaviour, ALLOWED_BEHAVIOUR_REFERENCES)

        # It's called globalFunctions for settings
        references["globalFunctions"] = references.pop("global_functions", {})

        return Rules_UpdateRules(behaviour=self.behaviour, references=references)

    def build_create_proto(self) -> Message:
        """Create a proto for creating the resource."""
        raise NotImplementedError("Create operation not supported for Rules settings.")

    def build_delete_proto(self) -> Message:
        """Create a proto for deleting the resource."""
        raise NotImplementedError("Delete operation not supported for Rules settings.")

    @property
    def command_type(self) -> str:
        """Get the update type for updating the resource."""
        return "rules"

    @staticmethod
    def discover_resources(base_path: str) -> list[str]:
        """Discover resources of this type in the given base path.

        Args:
            base_path (str): The base path to search for resources.

        Returns:
            list[str]: A list of file paths of discovered resources.
        """
        file_path = os.path.join(base_path, "agent_settings", "rules.txt")

        if not os.path.exists(file_path):
            return []

        return [file_path]



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class HandoffReferences(_message.Message):
    __slots__ = ("topics", "flow_steps")
    class TopicsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class FlowStepsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    TOPICS_FIELD_NUMBER: _ClassVar[int]
    FLOW_STEPS_FIELD_NUMBER: _ClassVar[int]
    topics: _containers.ScalarMap[str, bool]
    flow_steps: _containers.ScalarMap[str, bool]
    def __init__(self, topics: _Optional[_Mapping[str, bool]] = ..., flow_steps: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class SipConfig(_message.Message):
    __slots__ = ("invite", "refer", "bye")
    INVITE_FIELD_NUMBER: _ClassVar[int]
    REFER_FIELD_NUMBER: _ClassVar[int]
    BYE_FIELD_NUMBER: _ClassVar[int]
    invite: SipInviteHandoffConfig
    refer: SipReferHandoffConfig
    bye: SipByeHandoffConfig
    def __init__(self, invite: _Optional[_Union[SipInviteHandoffConfig, _Mapping]] = ..., refer: _Optional[_Union[SipReferHandoffConfig, _Mapping]] = ..., bye: _Optional[_Union[SipByeHandoffConfig, _Mapping]] = ...) -> None: ...

class SipInviteHandoffConfig(_message.Message):
    __slots__ = ("phone_number", "outbound_endpoint", "outbound_encryption")
    PHONE_NUMBER_FIELD_NUMBER: _ClassVar[int]
    OUTBOUND_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    OUTBOUND_ENCRYPTION_FIELD_NUMBER: _ClassVar[int]
    phone_number: str
    outbound_endpoint: str
    outbound_encryption: str
    def __init__(self, phone_number: _Optional[str] = ..., outbound_endpoint: _Optional[str] = ..., outbound_encryption: _Optional[str] = ...) -> None: ...

class SipReferHandoffConfig(_message.Message):
    __slots__ = ("phone_number",)
    PHONE_NUMBER_FIELD_NUMBER: _ClassVar[int]
    phone_number: str
    def __init__(self, phone_number: _Optional[str] = ...) -> None: ...

class SipByeHandoffConfig(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class SipHeaders(_message.Message):
    __slots__ = ("headers",)
    HEADERS_FIELD_NUMBER: _ClassVar[int]
    headers: _containers.RepeatedCompositeFieldContainer[SipHeader]
    def __init__(self, headers: _Optional[_Iterable[_Union[SipHeader, _Mapping]]] = ...) -> None: ...

class SipHeader(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

class Handoffs(_message.Message):
    __slots__ = ("handoffs",)
    HANDOFFS_FIELD_NUMBER: _ClassVar[int]
    handoffs: _containers.RepeatedCompositeFieldContainer[HandoffConfig]
    def __init__(self, handoffs: _Optional[_Iterable[_Union[HandoffConfig, _Mapping]]] = ...) -> None: ...

class HandoffConfig(_message.Message):
    __slots__ = ("id", "name", "created_by", "created_at", "description", "sip_config", "sip_headers", "active", "updated_by", "updated_at", "references", "is_default")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    SIP_CONFIG_FIELD_NUMBER: _ClassVar[int]
    SIP_HEADERS_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    IS_DEFAULT_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    created_by: str
    created_at: _timestamp_pb2.Timestamp
    description: str
    sip_config: SipConfig
    sip_headers: SipHeaders
    active: bool
    updated_by: str
    updated_at: _timestamp_pb2.Timestamp
    references: HandoffReferences
    is_default: bool
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., created_by: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., description: _Optional[str] = ..., sip_config: _Optional[_Union[SipConfig, _Mapping]] = ..., sip_headers: _Optional[_Union[SipHeaders, _Mapping]] = ..., active: bool = ..., updated_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., references: _Optional[_Union[HandoffReferences, _Mapping]] = ..., is_default: bool = ...) -> None: ...

class Handoff_Create(_message.Message):
    __slots__ = ("id", "name", "description", "sip_config", "sip_headers", "active", "references")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    SIP_CONFIG_FIELD_NUMBER: _ClassVar[int]
    SIP_HEADERS_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    description: str
    sip_config: SipConfig
    sip_headers: SipHeaders
    active: bool
    references: HandoffReferences
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., description: _Optional[str] = ..., sip_config: _Optional[_Union[SipConfig, _Mapping]] = ..., sip_headers: _Optional[_Union[SipHeaders, _Mapping]] = ..., active: bool = ..., references: _Optional[_Union[HandoffReferences, _Mapping]] = ...) -> None: ...

class Handoff_Update(_message.Message):
    __slots__ = ("id", "name", "description", "sip_config", "sip_headers", "active", "references")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    SIP_CONFIG_FIELD_NUMBER: _ClassVar[int]
    SIP_HEADERS_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    description: str
    sip_config: SipConfig
    sip_headers: SipHeaders
    active: bool
    references: HandoffReferences
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., description: _Optional[str] = ..., sip_config: _Optional[_Union[SipConfig, _Mapping]] = ..., sip_headers: _Optional[_Union[SipHeaders, _Mapping]] = ..., active: bool = ..., references: _Optional[_Union[HandoffReferences, _Mapping]] = ...) -> None: ...

class Handoff_SetDefault(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class Handoff_Delete(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class Handoff_ResetHandoffConfigs(_message.Message):
    __slots__ = ("sip_config", "sip_headers")
    SIP_CONFIG_FIELD_NUMBER: _ClassVar[int]
    SIP_HEADERS_FIELD_NUMBER: _ClassVar[int]
    sip_config: SipConfig
    sip_headers: SipHeaders
    def __init__(self, sip_config: _Optional[_Union[SipConfig, _Mapping]] = ..., sip_headers: _Optional[_Union[SipHeaders, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: translations.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12translations.proto\x12\x0ctranslations\x1a\x1fgoogle/protobuf/timestamp.proto\"P\n\rLocalizedText\x12\x15\n\rlanguage_code\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\x1a\n\x12is_auto_translated\x18\x03 \x01(\x08\"\x9a\x02\n\x10TranslationEntry\x12\n\n\x02id\x18\x01 \x01(\t\x12\x17\n\x0ftranslation_key\x18\x02 \x01(\t\x12\x31\n\x0ctranslations\x18\x03 \x03(\x0b\x32\x1b.translations.LocalizedText\x12.\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x33\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x17\n\nupdated_by\x18\x07 \x01(\tH\x01\x88\x01\x01\x42\r\n\x0b_updated_atB\r\n\x0b_updated_by\"D\n\x0cTranslations\x12\x34\n\x0ctranslations\x18\x01 \x03(\x0b\x32\x1e.translations.TranslationEntry\"x\n\x0bUpdateEntry\x12\x15\n\rlanguage_code\x18\x01 \x01(\t\x12\x11\n\x04text\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1f\n\x12is_auto_translated\x18\x03 \x01(\x08H\x01\x88\x01\x01\x42\x07\n\x05_textB\x15\n\x13_is_auto_translated\"x\n\x1eLanguageHubTranslations_Create\x12\n\n\x02id\x18\x01 \x01(\t\x12\x17\n\x0ftranslation_key\x18\x02 \x01(\t\x12\x31\n\x0ctranslations\x18\x03 \x03(\x0b\x32\x1b.translations.LocalizedText\"\x8f\x01\n\x1eLanguageHubTranslations_Update\x12\n\n\x02id\x18\x01 \x01(\t\x12\x1c\n\x0ftranslation_key\x18\x02 \x01(\tH\x00\x88\x01\x01\x12/\n\x0ctranslations\x18\x03 \x03(\x0b\x32\x19.translations.UpdateEntryB\x12\n\x10_translation_key\",\n\x1eLanguageHubTranslations_Delete\x12\n\n\x02id\x18\x01 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'translations_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_LOCALIZEDTEXT']._serialized_start=69
  _globals['_LOCALIZEDTEXT']._serialized_end=149
  _globals['_TRANSLATIONENTRY']._serialized_start=152
  _globals['_TRANSLATIONENTRY']._serialized_end=434
  _globals['_TRANSLATIONS']._serialized_start=436
  _globals['_TRANSLATIONS']._serialized_end=504
  _globals['_UPDATEENTRY']._serialized_start=506
  _globals['_UPDATEENTRY']._serialized_end=626
  _globals['_LANGUAGEHUBTRANSLATIONS_CREATE']._serialized_start=628
  _globals['_LANGUAGEHUBTRANSLATIONS_CREATE']._serialized_end=748
  _globals['_LANGUAGEHUBTRANSLATIONS_UPDATE']._serialized_start=751
  _globals['_LANGUAGEHUBTRANSLATIONS_UPDATE']._serialized_end=894
  _globals['_LANGUAGEHUBTRANSLATIONS_DELETE']._serialized_start=896
  _globals['_LANGUAGEHUBTRANSLATIONS_DELETE']._serialized_end=940
# @@protoc_insertion_point(module_scope)



"""API authentication and utility helpers

Copyright PolyAI Limited
"""


def clean_body(body: dict) -> dict:
    """Clean the body dictionary by removing None values
    as the API schema does not allow them.
    """
    cleaned = {}
    for k, v in body.items():
        if isinstance(v, dict):
            v = clean_body(v)
        if v is not None:
            cleaned[k] = v
    return cleaned


def filter_dict(dict: dict, required_keys: list[str]) -> dict:
    """Clean the dictionary by removing None values."""
    return {k: v for k, v in dict.items() if v is not None and k in required_keys}



"""Rich console output helpers for the ADK CLI.

Provides consistent, colorful terminal output with clean error formatting.

Copyright PolyAI Limited
"""

import json
import sys

from rich.console import Console
from rich.panel import Panel
from rich.syntax import Syntax
from rich.table import Table
from rich.theme import Theme

# Global verbose flag — set by CLI before commands run
_verbose = False

_theme = Theme(
    {
        "info": "cyan",
        "success": "green",
        "warning": "yellow",
        "error": "red bold",
        "filename.new": "green",
        "filename.modified": "green",
        "filename.deleted": "red",
        "filename.conflict": "red bold",
        "label": "bold",
        "muted": "dim",
    }
)

console = Console(theme=_theme, stderr=False)
err_console = Console(theme=_theme, stderr=True)


def set_verbose(verbose: bool) -> None:
    """Enable or disable verbose (traceback) output."""
    global _verbose
    _verbose = verbose


# ── Helpers ──────────────────────────────────────────────────────────


def success(message: str) -> None:
    console.print(f"[success]{message}[/success]")


def error(message: str) -> None:
    err_console.print(f"[error]Error:[/error] {message}")


def warning(message: str) -> None:
    console.print(f"[warning]Warning:[/warning] {message}")


def info(message: str) -> None:
    console.print(f"[info]{message}[/info]")


def plain(message: str) -> None:
    console.print(message)


# ── Structured output ────────────────────────────────────────────────


def print_status(
    region: str,
    account_id: str,
    project_id: str,
    last_updated: str,
    branch: str,
) -> None:
    """Print project status in a styled panel."""
    table = Table(show_header=False, box=None, padding=(0, 1))
    table.add_column("Key", style="label", no_wrap=True)
    table.add_column("Value")
    table.add_row("Region", region)
    table.add_row("Account ID", account_id)
    table.add_row("Project ID", project_id)
    table.add_row("Last Pulled", last_updated)
    table.add_row("Current Branch", branch)

    console.print(Panel(table, title="[bold]Project Status[/bold]", border_style="cyan"))


def print_file_list(title: str, files: list[str], style: str) -> None:
    """Print a labeled list of files in a given style."""
    if not files:
        return
    console.print(f"\n[label]{title}:[/label]")
    for f in files:
        console.print(f"  [{style}]{f}[/{style}]")


def print_diff(diff: str) -> None:
    """Print a unified diff with syntax highlighting."""
    console.print(Syntax(diff, "diff", theme="ansi_dark", line_numbers=False))


def print_branches(branches: dict[str, str] | list[str], current_branch: str | None) -> None:
    """Print branch list with current branch highlighted."""
    console.print("[label]Branches:[/label]")
    items = branches.keys() if isinstance(branches, dict) else branches
    for name in items:
        if name == current_branch:
            console.print(f"  [success]* {name}[/success] [muted](current)[/muted]")
        else:
            console.print(f"    {name}")


def print_validation_errors(errors: list[str]) -> None:
    """Print validation errors in a styled list."""
    console.print("[error]Project configuration is invalid.[/error]")
    for e in errors:
        console.print(f"  [error]-[/error] {e}")


def print_turn_metadata(
    response: dict,
    show_functions: bool = False,
    show_flow: bool = False,
    show_state: bool = False,
) -> None:
    """Print per-turn metadata above the agent response.

    Each section is opt-in via its corresponding flag:
      - show_functions: tool/function calls made this turn with their arguments
      - show_flow: the active flow and step name when the agent is inside a flow
      - show_state: variables added, updated, or removed this turn
    """
    if not (show_functions or show_flow or show_state):
        return

    metadata = response.get("metadata") or {}
    if not metadata:
        return

    function_events: list[dict] = metadata.get("function_events") or []
    in_flow: str | None = metadata.get("in_flow")
    in_step: str | None = metadata.get("in_step")

    # Aggregate state changes across every function event in this turn
    # (only needed when the state panel is enabled).
    all_added: dict = {}
    all_updated: dict = {}
    all_removed: list = []
    if show_state:
        for event in function_events:
            sc = event.get("state_changes") or {}
            all_added.update(sc.get("added") or {})
            all_updated.update(sc.get("updated") or {})
            all_removed.extend(sc.get("removed") or [])

    # ── FUNCTIONS ───────────────────────────────────────────────────────
    if show_functions and function_events:
        fn_table = Table(show_header=False, box=None, padding=(0, 0), expand=False)
        fn_table.add_column("call", overflow="fold")
        for event in function_events:
            name = event.get("name") or ""
            args = event.get("arguments") or {}
            if args:
                args_str = ", ".join(
                    f"{k}={json.dumps(v, ensure_ascii=False)}" for k, v in args.items()
                )
            else:
                args_str = ""
            fn_table.add_row(f"[bold cyan]{name}[/bold cyan]({args_str})")
        console.print(
            Panel(
                fn_table,
                title="[bold]Functions[/bold]",
                border_style="cyan",
                padding=(0, 1),
            )
        )

    # ── STATE CHANGES ───────────────────────────────────────────────────
    state_rows: list[tuple[str, str]] = []
    if show_state:
        for k, v in all_added.items():
            raw = v if isinstance(v, str) else json.dumps(v, ensure_ascii=False)
            if len(raw) > 120:
                raw = raw[:117] + "..."
            state_rows.append((f"[green]+ {k}[/green]", raw))
        for k, v in all_updated.items():
            value = v[-1] if isinstance(v, list) and v else v
            raw = value if isinstance(value, str) else json.dumps(value, ensure_ascii=False)
            if len(raw) > 120:
                raw = raw[:117] + "..."
            state_rows.append((f"[yellow]~ {k}[/yellow]", raw))
        for k in all_removed:
            state_rows.append((f"[red]- {k}[/red]", ""))

    if state_rows:
        state_table = Table(show_header=False, box=None, padding=(0, 1), expand=False)
        state_table.add_column("key", no_wrap=True)
        state_table.add_column("value", overflow="fold")
        for key_cell, val_cell in state_rows:
            state_table.add_row(key_cell, val_cell)
        console.print(
            Panel(
                state_table,
                title="[bold]State Changes[/bold]",
                border_style="yellow",
                padding=(0, 1),
            )
        )

    # ── FLOW / STEP ─────────────────────────────────────────────────────
    if show_flow and (in_flow or in_step):
        parts = []
        if in_flow:
            parts.append(f"Flow: [bold]{in_flow}[/bold]")
        if in_step:
            parts.append(f"Step: [bold]{in_step}[/bold]")
        console.print(
            Panel(
                "  |  ".join(parts),
                title="[bold]Flow / Step[/bold]",
                border_style="bright_magenta",
                padding=(0, 1),
            )
        )


# ── Error handling ───────────────────────────────────────────────────

# Maps exception types to user-friendly prefixes
_ERROR_MESSAGES: dict[type, str] = {
    FileNotFoundError: "File not found",
    ValueError: "Invalid value",
    OSError: "System error",
    ConnectionError: "Connection failed",
    TimeoutError: "Request timed out",
    ImportError: "Missing dependency",
}


def handle_exception(exc: Exception) -> None:
    """Print a clean error message, or full traceback in verbose mode."""
    if _verbose:
        err_console.print_exception(show_locals=False)
    else:
        # Try to find a user-friendly prefix
        prefix = None
        for exc_type, msg in _ERROR_MESSAGES.items():
            if isinstance(exc, exc_type):
                prefix = msg
                break

        # requests.HTTPError
        try:
            import requests

            if isinstance(exc, requests.HTTPError):
                prefix = "API request failed"
        except ImportError:
            pass

        if prefix:
            error(f"{prefix}: {exc}")
        else:
            error(str(exc))

        err_console.print("[muted]Run with --verbose for the full traceback.[/muted]")

    sys.exit(1)



"""Handling and managing Agent Studio Pronunciations (TTS Rules)

Copyright PolyAI Limited
"""

import os
from dataclasses import dataclass
from typing import ClassVar, Optional

import poly.resources.resource_utils as utils
from poly.handlers.protobuf.pronunciations_pb2 import (
    Pronunciations_CreatePronunciation,
    Pronunciations_DeletePronunciation,
    Pronunciations_UpdatePronunciation,
)
from poly.resources.resource import (
    MultiResourceYamlResource,
    ResourceMapping,
    _parse_multi_resource_path,
)


@dataclass
class Pronunciation(MultiResourceYamlResource):
    """Dataclass representing a TTS Rule"""

    regex: str
    replacement: str
    case_sensitive: bool
    language_code: str
    description: str
    position: int
    top_level_name: ClassVar[str] = "pronunciations"

    def __init__(
        self,
        *,
        resource_id: Optional[str] = None,
        name: str = "",
        regex: str = "",
        replacement: str = "",
        case_sensitive: bool = False,
        language_code: str = "",
        description: str = "",
        position: int = 0,
    ):
        self.resource_id = resource_id
        self.name = name
        self.regex = regex
        self.replacement = replacement
        self.case_sensitive = case_sensitive
        self.language_code = language_code
        self.description = description
        self.position = position

    @property
    def file_path(self) -> str:
        # pronunciation rules don't have names
        path_safe_name = utils.clean_name(str(self.position), lowercase=False)
        return os.path.join(
            "voice",
            "response_control",
            "pronunciations.yaml",
            self.top_level_name,
            path_safe_name,
        )

    def to_yaml_dict(self) -> dict:
        d = {
            "regex": self.regex,
            "replacement": self.replacement,
            "case_sensitive": self.case_sensitive,
            "language_code": self.language_code,
            "description": self.description,
        }
        return {k: v for k, v in d.items() if (v != "" or k == "replacement")}

    @classmethod
    def from_yaml_dict(
        cls, yaml_dict: dict, resource_id: str, name: str, **kwargs
    ) -> "Pronunciation":
        return cls(
            resource_id=resource_id,
            name=yaml_dict.get("name", ""),
            regex=yaml_dict.get("regex", ""),
            replacement=yaml_dict.get("replacement", ""),
            case_sensitive=yaml_dict.get("case_sensitive", False),
            language_code=yaml_dict.get("language_code", ""),
            description=(yaml_dict.get("description") or "").strip(),
            position=kwargs.get("position", -1),
        )

    @classmethod
    def to_pretty_dict(
        cls, d: dict, resource_mappings: list[ResourceMapping] = None, **kwargs
    ) -> dict:
        """Return the pretty dictionary."""
        return d

    @classmethod
    def read_from_file(cls, file_path: str) -> str:
        true_file_path, segments = _parse_multi_resource_path(file_path)
        top_level_name = segments[0]
        resource_clean_name = segments[-1]
        top_level_yaml_dict = cls._get_top_level_data(true_file_path)
        # list() preserves document order from the YAML file
        yaml_list = list(top_level_yaml_dict.get(top_level_name, []))

        if not isinstance(yaml_list, list):
            raise ValueError(f"Top level YAML data is not a list: {top_level_yaml_dict}")

        matching_resource = next(
            (r for i, r in enumerate(yaml_list) if i == int(resource_clean_name)),
            None,
        )
        if not matching_resource:
            raise FileNotFoundError(
                f"Resource with name {resource_clean_name} not found in {true_file_path}"
            )

        return utils.dump_yaml(matching_resource)

    @classmethod
    def read_local_resource(
        cls, file_path: str, resource_id: str, resource_name: str, **kwargs
    ) -> "Pronunciation":
        """Read local resource and derive position from the item's place in the collection."""
        true_file_path, segments = _parse_multi_resource_path(file_path)
        top_level_name = segments[0]
        resource_clean_name = segments[-1]
        top_level_yaml_dict = cls._get_top_level_data(true_file_path)
        # list() preserves document order from the YAML file
        yaml_list = list(top_level_yaml_dict.get(top_level_name, []))

        position = 0
        contents = None
        for i, item in enumerate(yaml_list):
            if i == int(resource_clean_name):
                position = i
                contents = utils.dump_yaml(item)
                break

        if not contents:
            raise FileNotFoundError(
                f"Resource with name {resource_clean_name} not found in {true_file_path}"
            )

        content = cls.from_pretty(contents, file_path=file_path, **kwargs)
        try:
            yaml_dict = utils.load_yaml(content) or {}
        except Exception as e:
            raise ValueError(f"Error loading YAML file: {file_path}") from e

        return cls.from_yaml_dict(
            yaml_dict, resource_id=resource_id, name="", position=position, **kwargs
        )

    @property
    def command_type(self) -> str:
        return "pronunciation"

    @property
    def delete_command_type(self) -> str:
        return "pronunciations_delete_pronunciation"

    @property
    def create_command_type(self) -> str:
        return "pronunciations_create_pronunciation"

    @property
    def update_command_type(self) -> str:
        return "pronunciations_update_pronunciation"

    def build_create_proto(self) -> Pronunciations_CreatePronunciation:
        return Pronunciations_CreatePronunciation(
            id=self.resource_id,
            regex=self.regex,
            replacement=self.replacement,
            case_sensitive=self.case_sensitive,
            language_code=self.language_code,
            description=self.description,
            position=self.position,
            name=self.name,
        )

    def build_update_proto(self) -> Pronunciations_UpdatePronunciation:
        return Pronunciations_UpdatePronunciation(
            id=self.resource_id,
            regex=self.regex,
            replacement=self.replacement,
            case_sensitive=self.case_sensitive,
            language_code=self.language_code,
            description=self.description,
            position=self.position,
            name=self.name,
        )

    def build_delete_proto(self) -> Pronunciations_DeletePronunciation:
        return Pronunciations_DeletePronunciation(
            id=self.resource_id,
        )

    def validate(self, **kwargs) -> None:
        if not self.regex:
            raise ValueError("Regex pattern is required")

    def save(
        self, base_path: str, format: bool = False, save_to_cache: bool = False, **kwargs
    ) -> None:
        """Save the resource; pronunciations are matched by position (index), not by name."""
        content = self.to_pretty(**kwargs)
        if format:
            content = self.format_resource(content, file_name=str(self.position))
        file_path = self.get_path(base_path)

        yaml_content = utils.load_yaml(content) or {}
        true_file_path, segments = _parse_multi_resource_path(file_path)
        resource_clean_name = segments[-1]
        if not os.path.exists(true_file_path):
            if not save_to_cache:
                self.save_to_file(f"{self.top_level_name}: []", true_file_path)
            else:
                self._file_cache.setdefault(true_file_path, (0.0, {self.top_level_name: []}))

        top_level_yaml_dict = self._get_top_level_data(true_file_path)
        yaml_list = list(top_level_yaml_dict.get(self.top_level_name, []))
        if not isinstance(yaml_list, list):
            raise ValueError(f"Top level YAML data is not a list: {top_level_yaml_dict}")

        try:
            matching_idx = int(resource_clean_name)
        except ValueError:
            yaml_list.append(yaml_content)
        else:
            if matching_idx < len(yaml_list):
                # if index within list, assign the value to that index
                yaml_list[matching_idx] = yaml_content
            else:
                # otherwise, just append to the end
                yaml_list.append(yaml_content)

        top_level_yaml_dict[self.top_level_name] = yaml_list
        self._update_cache_after_write(true_file_path, top_level_yaml_dict)
        if not save_to_cache:
            self.save_to_file(utils.dump_yaml(top_level_yaml_dict), true_file_path)

    @classmethod
    def delete_resource(cls, file_path: str, save_to_cache: bool = False) -> None:
        """Delete the resource from the given file path."""
        true_file_path, segments = _parse_multi_resource_path(file_path)
        top_level_name = segments[0]
        if not os.path.exists(true_file_path):
            return
        top_level_yaml_dict = cls._get_top_level_data(true_file_path)
        resource_clean_name = segments[-1]
        yaml_list = top_level_yaml_dict.get(top_level_name, [])

        # Pronunciations use list index (not name/position) for identification
        try:
            matching_idx = int(resource_clean_name)
        except ValueError:
            return
        if matching_idx < 0 or matching_idx >= len(yaml_list):
            return
        del yaml_list[matching_idx]
        top_level_yaml_dict[top_level_name] = yaml_list

        cls._update_cache_after_write(true_file_path, top_level_yaml_dict)
        if not save_to_cache:
            cls.save_to_file(utils.dump_yaml(top_level_yaml_dict), true_file_path)

    @staticmethod
    def discover_resources(base_path: str) -> list[str]:
        yaml_path = os.path.join(base_path, "voice", "response_control", "pronunciations.yaml")
        discovered: list[str] = []

        if not os.path.exists(yaml_path):
            return discovered

        yaml_dict = Pronunciation._get_top_level_data(yaml_path)
        # list() preserves document order from the YAML file
        items: list[dict] = list(yaml_dict.get("pronunciations", [])) if yaml_dict else []

        for i, item in enumerate(items):
            path_safe_name = utils.clean_name(str(i), lowercase=False)
            discovered.append(os.path.join(yaml_path, Pronunciation.top_level_name, path_safe_name))

        return discovered



from poly.handlers.protobuf import snapshot_pb2 as _snapshot_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class RagDatabase(_message.Message):
    __slots__ = ("embedding_model", "embeddings_map")
    class EmbeddingsMapEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bytes
        def __init__(self, key: _Optional[str] = ..., value: _Optional[bytes] = ...) -> None: ...
    EMBEDDING_MODEL_FIELD_NUMBER: _ClassVar[int]
    EMBEDDINGS_MAP_FIELD_NUMBER: _ClassVar[int]
    embedding_model: str
    embeddings_map: _containers.ScalarMap[str, bytes]
    def __init__(self, embedding_model: _Optional[str] = ..., embeddings_map: _Optional[_Mapping[str, bytes]] = ...) -> None: ...

class RealTimeConfiguration(_message.Message):
    __slots__ = ("schema", "config")
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    CONFIG_FIELD_NUMBER: _ClassVar[int]
    schema: _struct_pb2.Struct
    config: _struct_pb2.Struct
    def __init__(self, schema: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., config: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...

class Bundle(_message.Message):
    __slots__ = ("snapshot", "rt_configurations", "rag_database")
    class RtConfigurationsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: RealTimeConfiguration
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[RealTimeConfiguration, _Mapping]] = ...) -> None: ...
    SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    RT_CONFIGURATIONS_FIELD_NUMBER: _ClassVar[int]
    RAG_DATABASE_FIELD_NUMBER: _ClassVar[int]
    snapshot: _snapshot_pb2.Snapshot
    rt_configurations: _containers.MessageMap[str, RealTimeConfiguration]
    rag_database: RagDatabase
    def __init__(self, snapshot: _Optional[_Union[_snapshot_pb2.Snapshot, _Mapping]] = ..., rt_configurations: _Optional[_Mapping[str, RealTimeConfiguration]] = ..., rag_database: _Optional[_Union[RagDatabase, _Mapping]] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ASRSettings(_message.Message):
    __slots__ = ("barge_in", "latency_config", "updated_by", "updated_at")
    BARGE_IN_FIELD_NUMBER: _ClassVar[int]
    LATENCY_CONFIG_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    barge_in: bool
    latency_config: LatencyConfig
    updated_by: str
    updated_at: _timestamp_pb2.Timestamp
    def __init__(self, barge_in: bool = ..., latency_config: _Optional[_Union[LatencyConfig, _Mapping]] = ..., updated_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class LatencyConfig(_message.Message):
    __slots__ = ("interaction_style",)
    INTERACTION_STYLE_FIELD_NUMBER: _ClassVar[int]
    interaction_style: str
    def __init__(self, interaction_style: _Optional[str] = ...) -> None: ...

class ASRSettings_UpdateASRSettings(_message.Message):
    __slots__ = ("barge_in", "latency_config")
    BARGE_IN_FIELD_NUMBER: _ClassVar[int]
    LATENCY_CONFIG_FIELD_NUMBER: _ClassVar[int]
    barge_in: bool
    latency_config: LatencyConfig
    def __init__(self, barge_in: bool = ..., latency_config: _Optional[_Union[LatencyConfig, _Mapping]] = ...) -> None: ...



"""Handling and managing Agent Studio ASR Settings (barge-in, interaction style)

Copyright PolyAI Limited
"""

import logging
import os
from dataclasses import dataclass
from functools import cached_property

from google.protobuf.message import Message

from poly.handlers.protobuf.asr_settings_pb2 import (
    ASRSettings_UpdateASRSettings,
    LatencyConfig,
)
from poly.handlers.protobuf.channels_pb2 import VoiceChannel_UpdateASRSettings
from poly.resources.resource import ResourceMapping, YamlResource

logger = logging.getLogger(__name__)

VALID_INTERACTION_STYLES = {"balanced", "precise", "swift", "sonic", "turbo"}


@dataclass
class AsrSettings(YamlResource):
    """Resource class for managing ASR (speech recognition) settings"""

    barge_in: bool
    interaction_style: str

    def __init__(
        self,
        *,
        resource_id: str,
        name: str,
        barge_in: bool = False,
        interaction_style: str = "balanced",
    ):
        self.resource_id = resource_id
        self.name = name
        self.barge_in = barge_in
        self.interaction_style = interaction_style

    @cached_property
    def file_path(self) -> str:
        return os.path.join("voice", "speech_recognition", "asr_settings.yaml")

    def to_yaml_dict(self) -> dict:
        return {
            "barge_in": self.barge_in,
            "interaction_style": self.interaction_style,
        }

    @classmethod
    def from_yaml_dict(
        cls, yaml_dict: dict, resource_id: str, name: str, **kwargs
    ) -> "AsrSettings":
        interaction_style = yaml_dict.get("interaction_style", "balanced")
        if interaction_style == "turbo":
            # this one is turbo in the FE, but sonic in the BE
            interaction_style = "sonic"

        return cls(
            resource_id=resource_id,
            name=name,
            barge_in=yaml_dict.get("barge_in", False),
            interaction_style=interaction_style,
        )

    def validate(self, resource_mappings: list[ResourceMapping] = None, **kwargs) -> None:
        if not self.interaction_style and not self.barge_in:
            logger.warning(
                "Neither interaction style or barge-in setting detected. "
                "These will be reset to default on next push."
            )

        if self.interaction_style not in VALID_INTERACTION_STYLES:
            raise ValueError(
                f"Invalid interaction_style '{self.interaction_style}'. "
                f"Must be one of: {', '.join(VALID_INTERACTION_STYLES)}"
            )

    @property
    def command_type(self) -> str:
        return "voice_asr_settings"

    @property
    def update_command_type(self) -> str:
        return "voice_channel_update_asr_settings"

    def build_update_proto(self) -> VoiceChannel_UpdateASRSettings:
        return VoiceChannel_UpdateASRSettings(
            asr_settings=ASRSettings_UpdateASRSettings(
                barge_in=self.barge_in,
                latency_config=LatencyConfig(
                    interaction_style=self.interaction_style,
                ),
            ),
        )

    def build_create_proto(self) -> Message:
        raise NotImplementedError("Create operation not supported for ASR settings.")

    def build_delete_proto(self) -> Message:
        raise NotImplementedError("Delete operation not supported for ASR settings.")

    @staticmethod
    def discover_resources(base_path: str) -> list[str]:
        file_path = os.path.join(base_path, "voice", "speech_recognition", "asr_settings.yaml")
        if not os.path.exists(file_path):
            return []
        return [file_path]



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class VariablesMap(_message.Message):
    __slots__ = ("variables",)
    class VariablesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    variables: _containers.ScalarMap[str, bool]
    def __init__(self, variables: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class VariableReferences(_message.Message):
    __slots__ = ("functions", "delay_responses", "flow_steps", "flow_no_code_steps", "flow_functions", "topics", "behaviours", "greetings", "roles", "personalities", "sms", "start_functions", "end_functions")
    class FunctionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class DelayResponsesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class FlowStepsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class FlowNoCodeStepsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class FlowFunctionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class TopicsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class BehavioursEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class GreetingsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class RolesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class PersonalitiesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class SmsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class StartFunctionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    class EndFunctionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bool
        def __init__(self, key: _Optional[str] = ..., value: bool = ...) -> None: ...
    FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    DELAY_RESPONSES_FIELD_NUMBER: _ClassVar[int]
    FLOW_STEPS_FIELD_NUMBER: _ClassVar[int]
    FLOW_NO_CODE_STEPS_FIELD_NUMBER: _ClassVar[int]
    FLOW_FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    TOPICS_FIELD_NUMBER: _ClassVar[int]
    BEHAVIOURS_FIELD_NUMBER: _ClassVar[int]
    GREETINGS_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    PERSONALITIES_FIELD_NUMBER: _ClassVar[int]
    SMS_FIELD_NUMBER: _ClassVar[int]
    START_FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    END_FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    functions: _containers.ScalarMap[str, bool]
    delay_responses: _containers.ScalarMap[str, bool]
    flow_steps: _containers.ScalarMap[str, bool]
    flow_no_code_steps: _containers.ScalarMap[str, bool]
    flow_functions: _containers.ScalarMap[str, bool]
    topics: _containers.ScalarMap[str, bool]
    behaviours: _containers.ScalarMap[str, bool]
    greetings: _containers.ScalarMap[str, bool]
    roles: _containers.ScalarMap[str, bool]
    personalities: _containers.ScalarMap[str, bool]
    sms: _containers.ScalarMap[str, bool]
    start_functions: _containers.ScalarMap[str, bool]
    end_functions: _containers.ScalarMap[str, bool]
    def __init__(self, functions: _Optional[_Mapping[str, bool]] = ..., delay_responses: _Optional[_Mapping[str, bool]] = ..., flow_steps: _Optional[_Mapping[str, bool]] = ..., flow_no_code_steps: _Optional[_Mapping[str, bool]] = ..., flow_functions: _Optional[_Mapping[str, bool]] = ..., topics: _Optional[_Mapping[str, bool]] = ..., behaviours: _Optional[_Mapping[str, bool]] = ..., greetings: _Optional[_Mapping[str, bool]] = ..., roles: _Optional[_Mapping[str, bool]] = ..., personalities: _Optional[_Mapping[str, bool]] = ..., sms: _Optional[_Mapping[str, bool]] = ..., start_functions: _Optional[_Mapping[str, bool]] = ..., end_functions: _Optional[_Mapping[str, bool]] = ...) -> None: ...

class Variable(_message.Message):
    __slots__ = ("id", "name", "created_by", "created_at", "updated_by", "updated_at", "references")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_BY_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    created_by: str
    created_at: _timestamp_pb2.Timestamp
    updated_by: str
    updated_at: _timestamp_pb2.Timestamp
    references: VariableReferences
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., created_by: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_by: _Optional[str] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., references: _Optional[_Union[VariableReferences, _Mapping]] = ...) -> None: ...

class Variables(_message.Message):
    __slots__ = ("entities",)
    ENTITIES_FIELD_NUMBER: _ClassVar[int]
    entities: _containers.RepeatedCompositeFieldContainer[Variable]
    def __init__(self, entities: _Optional[_Iterable[_Union[Variable, _Mapping]]] = ...) -> None: ...

class Variable_Create(_message.Message):
    __slots__ = ("id", "name", "references")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    references: VariableReferences
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., references: _Optional[_Union[VariableReferences, _Mapping]] = ...) -> None: ...

class Variable_Update(_message.Message):
    __slots__ = ("id", "name", "references")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    REFERENCES_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    references: VariableReferences
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., references: _Optional[_Union[VariableReferences, _Mapping]] = ...) -> None: ...

class Variable_Delete(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...
